<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_GF_Access_Manager {

	/**
	 * Setup class
	 */
	public function setup() {
		// Filter the download to redirect regular download buttons of mail locked downloads
		add_filter( 'dlm_can_download', array( $this, 'check_access' ), 30, 2 );
	}

	/**
	 * Check if requester has access to download
	 *
	 * @param bool $has_access
	 * @param DLM_Download $download
	 *
	 * @return bool
	 */
	public function check_access( $has_access, $download ) {

		// Check if the Download is GF locked
		$meta = get_post_meta( $download->get_id(), DLM_GF_Constants::META_FORM, true );
		if ( '' === $meta || 0 == $meta ) {
			return $has_access;
		}

		// get unlock type
		$unlock_type = $this->get_unlock_type();

		// This is a Gravity Forms Download, set access default to false
		$has_access = false;

		// Check if COOKIE is set
		if ( isset( $_COOKIE[ DLM_GF_Constants::COOKIE ] ) ) {

			// JSON decode cookie data
			$cookie_data = json_decode( stripslashes( $_COOKIE[ DLM_GF_Constants::COOKIE ] ), true );

			// Check if decode went ok
			if ( null !== $cookie_data ) {

				// check for global access
				if ( 'global' === $unlock_type ) {

					// GF cookie exists and the unlock_type is set to global, grant access
					$has_access = true;
				}

				// Check cookie data
				if ( false === $has_access && $cookie_data['form_id'] == $meta ) {

					// check if unlock type is set per_form
					if ( 'per_form' === $unlock_type ) {
						// GF cookie exists and the unlock_type is set to global, grant access
						$has_access = true;
					}

					// check if the requested download is the same as set in cookie
					if ( false === $has_access && $cookie_data['download_id'] == $download->get_id() ) {

						// Form ID and Download ID match, check if there's a submission with this id
						$entry = GFAPI::get_entry( absint( $cookie_data['sub_id'] ) );

						// Check submission
						if ( ! is_wp_error( $entry ) ) {

							// Get the form
							$form = GFAPI::get_form( $entry['form_id'] );

							if ( ! is_wp_error( $form ) ) {

								// GF Handler
								$gf_handler = new DLM_GF_Gravity_Forms_Handler();

								// Get lead index
								$lead_id = $gf_handler->get_lead_index_of_dlm_field( $form );

								if ( isset( $entry[ $lead_id ] ) ) {

									// Download ID and download hash are combined here because we can't dynamically add entry data
									$download_id_and_hash = explode( '|', $entry[ $lead_id ] );

									// Get the download hash
									$download_hash = ( isset( $download_id_and_hash[1] ) ? $download_id_and_hash[1] : '' );

									// Check Download Hash
									if ( '' !== $download_hash && isset( $cookie_data['download_hash'] ) && $download_hash === $cookie_data['download_hash'] ) {

										// There it is, you've got access. Enjoy the file :)
										$has_access = true;

									}
								}
							}
						}
					}
				}
			}
		}

		return $has_access;
	}

	/**
	 * Get unlock type setting
	 *
	 * @return string
	 */
	public function get_unlock_type() {
		return get_option( 'dlm_gf_unlock_type', 'per_download' );
	}

}