<?php
/* @package   GFChart\GFChart_Image_Charts
 * @copyright 2018 gravity+
 * @license   GPL-2.0+
 * @since     1.0.0
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Class GFChart_Image_Charts
 *
 * @since  1.0.0
 *
 * @author Naomi C. Bush for gravity+ <support@gravityplus.pro>
 */
class GFChart_Image_Charts {

	/**
	 * @since  1.0.0
	 *
	 * @author Naomi C. Bush for gravity+ <support@gravityplus.pro>
	 */
	private $min_gf_version = '2.3';

	public $_doing_notification_message = false;

	/**
	 * GFChart_Image_Charts constructor.
	 *
	 * @since  1.0.0
	 *
	 * @author Naomi C. Bush for gravity+ <support@gravityplus.pro>
	 */
	public function __construct() {
	}

	/**
	 * @since  1.0.0
	 *
	 * @author Naomi C. Bush for gravity+ <support@gravityplus.pro>
	 */
	public function run() {

		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );

		add_action( 'admin_init', array( $this, 'admin_init' ) );

	}

	/**
	 * Automatic updates
	 *
	 * @since  1.0.0
	 *
	 * @author Naomi C. Bush for gravity+ <support@gravityplus.pro>
	 */
	public function admin_init() {

		if ( ! class_exists( 'EDD_SL_Plugin_Updater' ) ) {

			include( GFCHART_IMAGE_CHARTS_PATH . 'EDD_SL_Plugin_Updater.php' );

		}


		if ( class_exists( 'GFChart' ) ) {

			global $gfp_gfchart;

			$license_key = trim( $gfp_gfchart->get_addon_object()->get_plugin_setting( 'license_key' ) );

			if ( ! empty( $license_key ) ) {

				$edd_updater = new EDD_SL_Plugin_Updater( GFCHART_IMAGE_CHARTS_EDD_STORE_URL, GFCHART_IMAGE_CHARTS_FILE, array(
						'version' => GFCHART_IMAGE_CHARTS_CURRENT_VERSION,
						'license' => $license_key,
						'item_id' => GFCHART_IMAGE_CHARTS_EDD_ITEM_ID,
						'author'  => 'Mensard with gravity+',
						'url'     => home_url()
					)
				);

			}

		}

	}

	/**
	 * @since  1.0.0
	 *
	 * @author Naomi C. Bush for gravity+ <support@gravityplus.pro>
	 */
	public function plugins_loaded() {

		if ( class_exists( 'GFForms' ) && class_exists( 'GFChart' ) ) {

			//TODO add image charts API keys to plugin settings page

			add_filter( 'gform_enable_shortcode_notification_message', array( $this, 'gform_enable_shortcode_notification_message' ) );

			add_filter( 'pre_do_shortcode_tag', array( $this, 'pre_do_shortcode_tag' ), 10, 3 );

		}

	}

	public function gform_enable_shortcode_notification_message( $enable ) {

		$this->_doing_notification_message = true;


		return $enable;
	}

	/**
	 * @param bool | mixed   $short_circuit
	 * @param string         $tag
	 * @param array | string $attr
	 *
	 * @return mixed
	 */
	public function pre_do_shortcode_tag( $short_circuit, $tag, $attr ) {

		if ( ( 'gfchart' == $tag ) && $this->_doing_notification_message || doing_filter( 'gfpdf_pdf_html_output' ) ) {

			global $gfp_gfchart;

			$gfp_gfchart->get_addon_object()->log_debug( __METHOD__ );


			$params = shortcode_atts( array( 'id'                    => 0,
			                                 'goal'                  => 0,
			                                 'calc_filter_field'     => 0,
			                                 'calc_filter_condition' => 'is',
			                                 'calc_filter_value'     => ''
			), $attr );

			if ( empty( $params['id'] ) ) {

				$gfp_gfchart->get_addon_object()->log_error( 'Chart ID empty: returning empty string' );

				return '';

			}

			$gfp_gfchart->shortcode_args = $params;

			$post_id = $params['id'];

			$source_form_id = get_post_meta( $post_id, 'source_form', true );

			$gfchart_config = apply_filters( 'gfchart_config', get_post_meta( $post_id, '_gfchart_config', true ), $post_id, $source_form_id );

			$filters = GFChart_API::get_gfchart_filter_vars( $post_id, true );

			$data = array();

			$chart_type_settings = GFChart_API::get_chart_type_settings( $gfchart_config['chart_type'] );


			if ( is_callable( $chart_type_settings['data_retriever'] ) ) {

				$data = call_user_func( $chart_type_settings['data_retriever'], $gfchart_config, $source_form_id, $filters );

			}

			if ( empty( $data ) || ! is_array( $data ) ) {

				$gfp_gfchart->get_addon_object()->log_error( "No data found for chart ID {$post_id}. Returning empty string" );

				return '';

			}

			if ( 'calc' == $gfchart_config['chart_type'] ) {

				return GFChart_API::format_calc_chart_data( $data, $gfchart_config )[0];

			}

			if ( is_callable( $chart_type_settings['data_formatter'] ) ) {

				$data = call_user_func( $chart_type_settings['data_formatter'], $data, $gfchart_config );

			}

			$chart_options = GFChart_API::format_chart_options( $gfchart_config );

			if ( empty( $chart_options['width'] ) ) {

				$chart_options['width'] = 400;
			}

			if ( empty( $chart_options['height'] ) ) {

				$chart_options['height'] = 300;
			}

			if ( false !== strpos( $chart_options['width'], '%' ) ) {

				$chart_options['width'] = 400;
			}

			$chart_object_info = array(
				'chart_type' => ucfirst( $gfchart_config['chart_type'] ),
				'chart_id'         => $post_id,
				'chart_data'       => $data,
				'chart_options'    => $chart_options
			);

			global $gfp_gfchart;

			$license_key = trim( $gfp_gfchart->get_addon_object()->get_plugin_setting( 'license_key' ) );

			$request = $this->make_request( $license_key, $chart_object_info );

			if ( $request['success'] ) {

				return "<img src=\"{$request['response']['url']}\" style=\"width: 50%;\" />";

			}

			return '';

		}


		return $short_circuit;
	}

	public function make_request( $api_key, $parameters ) {

		global $gfp_gfchart;

		$gfp_gfchart->get_addon_object()->log_debug( __METHOD__ );

		$response = array();

		$success = false;


		$api_url = "https://api.imagecharts.io/image/{$api_key}";

		$arguments = array(
			'timeout'      => 30,
			'sslverify'    => false,
			'headers'      => array( 'Content-Type' => 'application/json' ),
			'body'         => json_encode($parameters)
		);


		$raw_response = wp_remote_post( $api_url, $arguments );

		$response = json_decode( wp_remote_retrieve_body( $raw_response ), true );

		$gfp_gfchart->get_addon_object()->log_debug( 'Response: ' . print_r( $response, true) );

		if ( is_wp_error( $raw_response ) || ( 200 !== wp_remote_retrieve_response_code( $raw_response ) ) ) {

			return array( 'success' => $success, 'response' => $response );

		}

		$success = true;


		return array( 'success' => $success, 'response' => $response );

	}

}