<?php
/**
 * @package   Gravity_Forms_Timed_Entries
 * @author    PluginOwl <info@pluginowl.com>
 * @link      https://codecanyon.net/user/pluginowl/portfolio
 * @Copyright: 2020 Plugin Owl
 *
 * @wordpress-plugin
 * Plugin Name: Gravity Forms Timed Entries
 * Plugin URI: https://ccodecanyon.net/user/pluginowl/portfolio
 * Description: Extends Gravity Forms with entry timing and time restriction options
 * Version: 2.1
 * Author: Plugin Owl
 * Author URI: https://codecanyon.net/user/pluginowl/portfolio
 * Copyright: 2020 Plugin Owl
 */

//working on access only once per user
//apply lockedform prerender
//customize clock displays
//individual page timers - non ajax

// Prevent direct access to this file.
if (!defined('ABSPATH')) {
    header('HTTP/1.0 403 Forbidden');
    echo '404 File Not Found';
    exit; // Exit if accessed directly
}
// Redundancy with alt instruction
defined('ABSPATH') or die();

//prevent redundant loading if WP is loaded more than once
if (defined('GFTE_PLUGIN_FILE')) {
	exit;
}

// Main file path
define( 'GFTE_PLUGIN_FILE', __FILE__ );

// Set variable for admin page url
define('GFTE_ADMIN', admin_url('options-general.php?page=gravity-forms-timed-entries'));

// ADD 'Settings' Link
function gfte_settings_link($links, $file) {
    if($file == plugin_basename(GFTE_PLUGIN_FILE) && function_exists('admin_url') ) {
        $settings_link = '<a href="' . GFTE_ADMIN . '">' . __('Settings') . '</a>';
        array_unshift( $links, $settings_link );
    }
    return $links;
}
add_filter('plugin_action_links', 'gfte_settings_link', 10, 2);

//GFORMS TIMED ENTRIES JS LOAD
add_action('gform_enqueue_scripts', 'enqueue_gfte_custom_script', 30, 2);
function enqueue_gfte_custom_script($form, $is_ajax) {
	wp_enqueue_script('gfte_custom_script', plugin_dir_url(__FILE__)  . 'js/gravity-forms-timed-entries.js');
}

//GFORM TIMED ENTRIES FORM TIMER OPTIONS
class gfte_timed_entries_form_options {
	
    function __construct() {
        if(!property_exists('GFCommon', 'version') || !version_compare(GFCommon::$version, '1.8.5.8', '>='))
            return;
        add_filter('gform_tooltips', array($this, 'add_gfte_timed_entries_tooltips'));
        add_filter('gform_form_settings', array($this, 'add_gfte_timed_entries_setting'), 10, 2);
        add_action('gform_pre_form_settings_save', array($this, 'save_gfte_timed_entries_setting'), 10);
    }

    function add_gfte_timed_entries_tooltips($tooltips) {
		$tooltips['gfteshowtimertooltip'] 	= "<h6>" . __( "Show Main Timer", "gravityforms" ) . "</h6>" . __( "When enabled, the timer will be shown at the top of the form entry page.", "gravityforms" );
		$tooltips['gftecountdowntimertooltip'] 	= "<h6>" . __( "Countdown Timer", "gravityforms" ) . "</h6>" . __( "The main form timer counts down from the time limit instead of up. This requires a time limit to be set on the form.","gravityforms" );
		$tooltips['gftetimelimittooltip'] 	= "<h6>" . __( "Time Limit", "gravityforms" ) . "</h6>" . __( "Enter the total amount of HOURS, MINUTES, and SECONDS the user has to complete the form. Use only numerical input. Delete the option entries and update the form to remove any time limit.","gravityforms" );
        $tooltips['gfteclockendmessagetooltip'] 	= "<h6>" . __( "Clock End Message", "gravityforms" ) . "</h6>" . __( "This message will display in place of ALL entry timer countdown fields and the main countdown timer when the time limit is reached.</p><p>Default: Time Limit Reached.</p><p>This requires a time limit to be set on the form.","gravityforms" );
		$tooltips['gftetimelimittooltip'] 	= "<h6>" . __( "Time Limit", "gravityforms" ) . "</h6>" . __( "Enter the total amount of HOURS, MINUTES, and SECONDS the user has to complete the form. Use only numerical input. Delete the option entries and update the form to remove any time limit.","gravityforms" );
		$tooltips['gftetimesynchintervaltooltip'] 	= "<h6>" . __( "Synch Interval", "gravityforms" ) . "</h6>" . __( "<p>Enter how often the timer should check the server for an accurate time to avoid users getting around the local timer by changing their system clock. The MINIMUM time is 5 seconds. Setting to 0 or deleting the setting and saving will shut off server checks. Use only numerical input.</p><p><span style='color:red;'>NOTICE:</span> You should consider the potential load on your server for how often you run this check and how many people could be using the form at a time.</p>","gravityforms" );
		$tooltips['gfteblockedittooltip'] 	= "<h6>" . __( "Block Editing", "gravityforms" ) . "</h6>" . __( "<p>This option will disallow editing of the form once the time limit is reached. A time limit must be set for this form to use this option. This is intended for single page forms but will work on multi page form with a very small delay on page load before editing is blocked.<p>It is recommended that you give the user details on what the time limit is and what will happen when the time is out using the 'Time is Up Message' alert setting on the form settings page, an HTML field, the form description, or elsewhere on the form page.</p><p><b>You can select to keep individual fields editable after editing is blocked in the fields advanced tab within the form editor under 'Timed Entries'. This allows you to let the user correct specific required fields or fields which could fail validation if you are using blocked editing on a form with open entry fields.</b></p><p><span style='color:red;'>NOTICE:</span> This should NOT be used when any fields are required or the user could potentially enter invalid data in a field which will not pass validation because they will not be able to edit the data failing validation to submit it properly.</p>","gravityforms" );
        $tooltips['gftetimerstartpagetooltip'] 	= "<h6>" . __( "Timing Start Page", "gravityforms" ) . "</h6>" . __( "<p>This option will hide and delay the start of all form timers until the input form page number is visited.</p><p>The input page number to start on must be 2 or higher.</p><p>Delete the page number to return to starting timers from the initial form first page load.</p>","gravityforms" );
		$tooltips['gftetimesupmessagetooltip'] 	= "<h6>" . __( "Time is Up Message", "gravityforms" ) . "</h6>" . __( "<p>Enter what you would like to display to the user as an onscreen alert when the time limit has been reached. This message will NOT display if form is being auto submitted. Removing the message here will remove any alert on reching the time limit. </p>","gravityforms" );
        $tooltips['gfteautoredirecttooltip'] 	= "<h6>" . __( "Auto Redirect URL", "gravityforms" ) . "</h6>" . __( "<p>This option will automatically hide the form and redirect the form page to this URL without submitting the form once the time limit is reached. <b>You cannot Auto Submit and redirect at the same time.</b> You CAN Auto Submit and use the form's confirmation page redirect instead. A time limit must be set for this form to use this option.</p><p>Delete the URL to remove the redirect.</p><p>It is recommended that you give the user details on what the time limit is and what will happen when the time is out using the 'Time is Up Message' alert setting on the form settings page, an HTML field, the form description, or elsewhere on the form page.</p>","gravityforms" );
		$tooltips['gfteautosubmittooltip'] 	= "<h6>" . __( "Auto Submit", "gravityforms" ) . "</h6>" . __( "<p>This option will automatically submit the form once the time limit is reached. A time limit must be set for this form to use this option. This is intended for single page forms but will work on multi page forms with each remaining page being submitted in succession until the form is submitted.</p><p>It is recommended that you give the user details on what the time limit is and what will happen when the time is out using the 'Time is Up Message' alert setting on the form settings page, an HTML field, the form description, or elsewhere on the form page.</p><p><span style='color:red;'>NOTICE:</span> This option removes <b>ALL</b> data validation (incliduing required fields) and is intended for quizes or other forms where the user can only select from valid choices and does not have free form data entry that could be invalid or maliciously submitted.</p>","gravityforms" );
        $tooltips['gftecustomtimeseparatortooltip'] 	= "<h6>" . __( "Custom Time Text", "gravityforms" ) . "</h6>" . __( "<p>Change 'Hours', 'Minutes', and 'Seconds' to custom text.</p>","gravityforms" );
        $tooltips['gftetimedeventstooltip'] 	= "<h6>" . __( "Additional Timed Events", "gravityforms" ) . "</h6>" . __( "<p>Enter custom timed events.</p><p>Available events: block, hide, show, alert. </p><p>See plugin <a href='/wp-admin/options-general.php?page=gravity-forms-timed-entries'>settings page</a> for instructions.</p>","gravityforms" );
        $tooltips['gftemaintimertexttooltip'] 	= "<h6>" . __( "Main Timer Text", "gravityforms" ) . "</h6>" . __( "<p>Display custom text below the main form timer.</p>","gravityforms" );
        return $tooltips;
    }

    function add_gfte_timed_entries_setting($settings, $form) {
		$gfte_show_timer 			= rgar($form, 'gfteshowtimer') ? 'checked="checked"' : "";
        $gfte_main_timer_text       = rgar($form, 'gftemaintimertext') ? rgar($form, 'gftemaintimertext') : "";
		$gfte_countdown_timer 		= rgar($form, 'gftecountdowntimer') ? 'checked="checked"' : "";
        $gfte_clock_end_message		= rgar($form, 'gfteclockendmessage') ? rgar($form, 'gfteclockendmessage') : "";
		$gfte_time_limit 			= rgar($form, 'gftetimelimit') ? rgar($form, 'gftetimelimit') : "";
		$gfte_time_limit_sec		= rgar($form, 'gftetimelimitsec') ? rgar($form, 'gftetimelimitsec') : "";
		$gfte_time_limit_min		= rgar($form, 'gftetimelimitmin') ? rgar($form, 'gftetimelimitmin') : "";
		$gfte_time_limit_hour		= rgar($form, 'gftetimelimithour') ? rgar($form, 'gftetimelimithour') : "";
		$gfte_time_synch_interval	= rgar($form, 'gftetimesynchinterval') ? rgar($form, 'gftetimesynchinterval') : "";
        $gfte_timer_start_page	    = rgar($form, 'gftetimerstartpage') ? rgar($form, 'gftetimerstartpage') : "";
		$gfte_block_edit			= rgar($form, 'gfteblockediting') ? 'checked="checked"' : "";
		$gfte_times_up_message		= rgar($form, 'gftetimesupmessage') ? rgar($form, 'gftetimesupmessage') : "";
        $gfte_auto_redirect			= rgar($form, 'gfteautoredirect') ? rgar($form, 'gfteautoredirect') : "";
		$gfte_auto_submit			= rgar($form, 'gfteautosubmit') ? 'checked="checked"' : "";
        $gfte_custom_time_sep_hours	= rgar($form, 'gftecustomtimeseparatorhours') ? rgar($form, 'gftecustomtimeseparatorhours') : "";
        $gfte_custom_time_sep_mins	= rgar($form, 'gftecustomtimeseparatormins') ? rgar($form, 'gftecustomtimeseparatormins') : "";
        $gfte_custom_time_sep_secs	= rgar($form, 'gftecustomtimeseparatorsecs') ? rgar($form, 'gftecustomtimeseparatorsecs') : "";
        $gfte_timed_events          = rgar($form, 'gftetimedevents') ? rgar($form, 'gftetimedevents') : "";
		
		$gfte_options_available		= false;
		foreach ($form['fields'] as $field) {
			if ($field->type == 'gftetimer') {
				$gfte_options_available	= true;
				break;
			}
		}
		
		$gfte_time_limit_notice = null;
		if ($gfte_time_synch_interval && $gfte_time_limit) {
			$gfte_server_wait = $gfte_time_synch_interval - floor($gfte_time_limit % $gfte_time_synch_interval);
			$gfte_server_wait = '<br/><i>Synch Interval will run within ' . $gfte_server_wait . ' seconds of time limit.</i>';
		}
		if ($gfte_time_limit && ($gfte_time_limit_sec || $gfte_time_limit_min || $gfte_time_limit_hour)) {
			
			$gfte_time_limit_notice = '<i>Time limit = ' . $gfte_time_limit . ' seconds.</i>' . $gfte_server_wait;
		}
		else if ($gfte_time_limit && !$gfte_time_limit_sec && !$gfte_time_limit_min && !$gfte_time_limit_hour) {
			$gfte_time_limit_notice = '<i>Time limit from previous plugin version currently set to ' . $gfte_time_limit . ' seconds. Please update using settings above for hours/minutes/seconds.</i>' . $gfte_server_wait;
		} else {
			$gfte_time_limit_notice = 'Time limit not set.';
		}
		
		
		if ($gfte_options_available	=== true) {
			$settings['Form Timed Entries']['gftetimedentries'] = '
				<tr>
					<th><img src="' . plugin_dir_url(__FILE__)  . 'images/owl-small.png" height="12" width="12" style="display:inline-block; float:left; margin-top:4px;">
						<a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank"> Plugin Owl</a></th>
					<td>
						<p><b>To use these form timer options you must keep an entry timer field in the form somewhere.<br/>The entry timer field can be hidden and/or excluded from the entry if desired.</b></p>
					</td>
				</tr>
				<tr>
					<th>' . __("Show Main Timer", "gravityforms") . ' ' . gform_tooltip("gfteshowtimertooltip", "", true) . '</th>
					<td>
						<input type="checkbox" id="gfteshowtimer" name="gfteshowtimer" value="1" ' . $gfte_show_timer . ' />
						<label for="gfteshowtimer">' . __("Show a main form timer on top of the form", "gravityforms") . '</label>
					</td>
				</tr>
                <tr>
					<th>' . __("Main Timer Text", "gravityforms") . ' ' . gform_tooltip("gftemaintimertexttooltip", "", true) . '</th>
					<td>
						<textarea id="gftemaintimertext" name="gftemaintimertext" value=" placeholder="custom main timer text" style="height:100px; max-width: 550px; width: 90%"/>' . $gfte_main_timer_text . '</textarea><br/>
						<label for="gftemaintimertext">' . __("Custom main timer text.", "gravityforms") . '</label>
					</td>
				</tr>
				<tr>
					<th>' . __("Countdown Timer", "gravityforms") . ' ' . gform_tooltip("gftecountdowntimertooltip", "", true) . '</th>
					<td>
						<input type="checkbox" id="gftecountdowntimer" name="gftecountdowntimer" value="1" ' . $gfte_countdown_timer . ' />
						<label for="gftecountdowntimer">' . __("Main timer counts down", "gravityforms") . '</label>
					</td>
				</tr>
                <tr>
					<th>' . __("Clock End Message", "gravityforms") . ' ' . gform_tooltip("gfteclockendmessagetooltip", "", true) . '</th>
					<td>
						<textarea id="gfteclockendmessage" name="gfteclockendmessage" style="height:100px; max-width:550px; width:90%;" placeholder="Time is Up!">' . $gfte_clock_end_message . '</textarea><br/>
						<label for="gfteclockendmessage">' . __("Custom countdown clock text when the time limit has been reached.", "gravityforms") . '</label>
					</td>
				</tr>
				<tr>
					<th>' . __("Time limit", "gravityforms") . ' ' . gform_tooltip("gftetimelimittooltip", "", true) . '</th>
					<td>
						<input type="number" id="gftetimelimithour" name="gftetimelimithour" onkeypress="return event.charCode >= 48 && event.charCode <= 57" oninput="this.value=this.value.replace(/[^0-9]/g,' . "''" . ');" value="' . $gfte_time_limit_hour . '" step="60"/>
						<label for="gftetimelimithour">' . __("Hours", "gravityforms") . '</label>
						<input type="number" id="gftetimelimitmin" name="gftetimelimitmin" onkeypress="return event.charCode >= 48 && event.charCode <= 57" oninput="this.value=this.value.replace(/[^0-9]/g,' . "''" . ');" value="' . $gfte_time_limit_min . '" step="60"/>
						<label for="gftetimelimitmin">' . __("Minutes", "gravityforms") . '</label>
						<input style="display:inline-block;" type="number" id="gftetimelimitsec" name="gftetimelimitsec" onkeypress="return event.charCode >= 48 && event.charCode <= 57" oninput="this.value=this.value.replace(/[^0-9]/g,' . "''" . ');" value="' . $gfte_time_limit_sec . '" step="60"/>
						<label for="gftetimelimitsec">' . __("Seconds", "gravityforms") . '</label><br/>
						' . $gfte_time_limit_notice . '
					</td>
				</tr>
				<tr>
					<th>' . __("Synch Interval", "gravityforms") . ' ' . gform_tooltip("gftetimesynchintervaltooltip", "", true) . '</th>
					<td>
						<input type="number" id="gftetimesynchinterval" name="gftetimesynchinterval" onkeypress="return event.charCode >= 48 && event.charCode <= 57" oninput="this.value=this.value.replace(/[^0-9]/g,' . "''" . ');" onblur="if (value < 5 && value != 0){value = 5};" value="' . $gfte_time_synch_interval . '" step="5"/><br/>
						<label for="gftetimesynchinterval">' . __("Enter the time between server time checks in seconds (Minimum is 5). <b>Read tooltip!</b>", "gravityforms") . '</label>
					</td>
				</tr>
                <tr>
					<th>' . __("Timing Start Page", "gravityforms") . ' ' . gform_tooltip("gftetimerstartpagetooltip", "", true) . '</th>
					<td>
						<input type="number" id="gftetimerstartpage" name="gftetimerstartpage" onkeypress="return event.charCode >= 48 && event.charCode <= 57" oninput="this.value=this.value.replace(/[^0-9]/g,' . "''" . ');" value="' . $gfte_timer_start_page . '" /><br/>
						<label for="gftetimerstartpage">' . __("Start entry timers when this form page number is visited.", "gravityforms") . '</label>
					</td>
				</tr>
				<tr>
					<th>' . __("Block Editing", "gravityforms") . ' ' . gform_tooltip("gfteblockedittooltip", "", true) . '</th>
					<td>
						<input type="checkbox" id="gfteblockediting" name="gfteblockediting" value="1" ' . $gfte_block_edit . ' />
						<label for="gfteblockediting">' . __("Block editing of the form once the time limit is reached. <b>Read tooltip!</b>", "gravityforms") . '</label>
					</td>
				</tr>
				<tr>
					<th>' . __("Time is Up Message", "gravityforms") . ' ' . gform_tooltip("gftetimesupmessagetooltip", "", true) . '</th>
					<td>
						<textarea id="gftetimesupmessage" name="gftetimesupmessage" style="height:100px; max-width:550px; width:90%;" placeholder="The Time Limit has been Reached!">' . $gfte_times_up_message . '</textarea><br/>
						<label for="gftetimesupmessage">' . __("Custom pop-up message when the time limit has been reached.", "gravityforms") . '</label>
					</td>
				</tr>
                <tr>
					<th>' . __("Auto Redirect URL", "gravityforms") . ' ' . gform_tooltip("gfteautoredirecttooltip", "", true) . '</th>
					<td>
						<input type="text" id="gfteautoredirect" name="gfteautoredirect" value="' . $gfte_auto_redirect . '" placeholder="http://redirect-URL.com" style="max-width: 550px; width: 90%;"/><br/>
						<label for="gfteautoredirect">' . __("Load this URL to the page when time limit is reached. <b>Read tooltip!</b>", "gravityforms") . '</label>
					</td>
				</tr>
				<tr>
					<th>' . __("Auto Submit", "gravityforms") . ' ' . gform_tooltip("gfteautosubmittooltip", "", true) . '</th>
					<td>
						<input type="checkbox" id="gfteautosubmit" name="gfteautosubmit" value="1" ' . $gfte_auto_submit . ' />
						<label for="gfteautosubmit">' . __("Auto submit the form when time limit is reached. <b>Read tooltip!</b>", "gravityforms") . '</label>
					</td>
				</tr>
                <tr>
					<th>' . __("Custom Time Text", "gravityforms") . ' ' . gform_tooltip("gftecustomtimeseparatortooltip", "", true) . '</th>
					<td>
						Hours = <input type="text" id="gftecustomtimeseparatorhours" name="gftecustomtimeseparatorhours" value="' . $gfte_custom_time_sep_hours . '" placeholder="custom hours text" style="max-width: 550px; display: inline-block;"/> Minutes = <input type="text" id="gftecustomtimeseparatormins" name="gftecustomtimeseparatormins" value="' . $gfte_custom_time_sep_mins . '" placeholder="custom minutes text" style="max-width: 550px; display: inline-block;"/> Seconds = <input type="text" id="gftecustomtimeseparatorsecs" name="gftecustomtimeseparatorsecs" value="' . $gfte_custom_time_sep_secs . '" placeholder="custom seconds text" style="max-width: 550px; display: inline-block;"/>
					</td>
				</tr>
                <tr>
					<th>' . __("Additional Timed Events", "gravityforms") . ' ' . gform_tooltip("gftetimedeventstooltip", "", true) . '</th>
					<td>
						<textarea id="gftetimedevents" name="gftetimedevents" style="height:100px; max-width:550px; width:90%;" placeholder="event type::CSS class or alert message::seconds to activate on">' . $gfte_timed_events . '</textarea><br/>
						<label for="gftetimedevents">' . __("Custom timed events. See plugin <a href='/wp-admin/options-general.php?page=gravity-forms-timed-entries'>settings page</a> for instructions.", "gravityforms") . '</label>
					</td>
				</tr>';
		} else {
			$settings['Form Timed Entries']['gftetimedentries'] = '
				<tr>
					<th><img src="' . plugin_dir_url(__FILE__)  . 'images/owl-small.png" height="12" width="12" style="display:inline-block; float:left; margin-top:4px;">
						<a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank"> Plugin Owl</a></th>
					<td>
						<p><b>To use form time limit options you must include an entry timer field in the form somewhere.<br/>The entry timer field is added from the advanced fields, and can be hidden and/or excluded from the entry if desired.</b></p>
					</td>
				</tr>';
		}
		return $settings;
    }
	
    function save_gfte_timed_entries_setting($form) {
		$form['gfteshowtimer'] 			            = rgpost('gfteshowtimer');
        $form['gftemaintimertext'] 			        = rgpost('gftemaintimertext');
		$form['gftecountdowntimer'] 	            = rgpost('gftecountdowntimer');
        $form['gfteclockendmessage'] 	            = rgpost('gfteclockendmessage');
		$form['gftetimelimit'] 			            = (intval(rgpost('gftetimelimithour')) * 3600) + (intval(rgpost('gftetimelimitmin')) * 60) + intval(rgpost('gftetimelimitsec'));
		$form['gftetimelimitsec'] 		            = intval(rgpost('gftetimelimitsec'));
		$form['gftetimelimitmin'] 		            = intval(rgpost('gftetimelimitmin'));
		$form['gftetimelimithour'] 		            = rgpost('gftetimelimithour');
		if (intval(rgpost('gftetimesynchinterval')) >= 5){
			$form['gftetimesynchinterval'] 	        = rgpost('gftetimesynchinterval');
		} else {
			$form['gftetimesynchinterval'] 	        = '';
		}
        if (intval(rgpost('gftetimerstartpage')) <= 1){
			$form['gftetimerstartpage'] 	        = '';
		} else {
			$form['gftetimerstartpage'] 	        = rgpost('gftetimerstartpage');
		}
		$form['gfteblockediting'] 		            = rgpost('gfteblockediting');
		$form['gftetimesupmessage'] 	            = rgpost('gftetimesupmessage');
        $form['gfteautoredirect'] 		            = rgpost('gfteautoredirect');
		$form['gfteautosubmit'] 		            = rgpost('gfteautosubmit');
        $form['gftecustomtimeseparatorhours'] 		= rgpost('gftecustomtimeseparatorhours');
        $form['gftecustomtimeseparatormins'] 		= rgpost('gftecustomtimeseparatormins');
        $form['gftecustomtimeseparatorsecs'] 		= rgpost('gftecustomtimeseparatorsecs');
        $form['gftetimedevents'] 	                = rgpost('gftetimedevents');
		//remove block edit and autosubmit if no time limit is set
		if (!$form['gftetimelimit']) {
			$form['gfteblockediting'] = false;
			$form['gfteautoredirect'] = false;
            $form['gfteautosubmit'] = false;
		}
        return $form;
    }
}
new gfte_timed_entries_form_options();

//GFORM TIMED ENTRIES DISPLAY FORM OPTION FILTER
add_filter('gform_get_form_filter', 'gfte_main_form_filter', 10, 2);
function gfte_main_form_filter($form_string, $form) {
	$form_string = '
		<style type="text/css">
			.gfte-show, .gfte-hidden {
				visibility: hidden;
				display: none !important;
			}
		</style>' . $form_string;
	if (isset($form['gfteshowtimer']) && $form['gfteshowtimer']) {
        $gftecustomHours    = $form['gftecustomtimeseparatorhours'] ? $form['gftecustomtimeseparatorhours'] : "Hours";
        $gftecustomMins     = $form['gftecustomtimeseparatormins'] ? $form['gftecustomtimeseparatormins'] : "Minutes";
        $gftecustomSecs     = $form['gftecustomtimeseparatorsecs'] ? $form['gftecustomtimeseparatorsecs'] : "Seconds";
        $gftemainTimerText  = $form['gftemaintimertext'] ? $form['gftemaintimertext'] : false;
        $gfteAddTimerText   = $gftemainTimerText ? '<br/><span class="gfteMainTimerText">' . esc_html__($gftemainTimerText) . '</span>' : '';
		$gfteClockType	    = '<p class="gfteCountupTimer"><span class="gftehours">0</span><span class="gftehoursseparator"> ' . esc_html__($gftecustomHours) . ' </span><span class="gfteminutes">0</span><span class="gfteminsseparator"> ' . esc_html__($gftecustomMins) . ' </span><span class="gfteseconds">0</span><span class="gftesecondssseparator"> ' . esc_html__($gftecustomSecs) . '</span>' . $gfteAddTimerText . '</p>';
		if ($form['gftecountdowntimer']) {
			$gfteClockType	= '<p class="gfteCountdownTimer"><span class="gfte-hours">0</span><span class="gfte-hoursseparator"> ' . esc_html__($gftecustomHours) . ' </span><span class="gfte-minutes">0</span><span class="gfte-minsseparator"> ' . esc_html__($gftecustomMins) . ' </span><span class="gfte-seconds">0</span><span class="gfte-secondssseparator"> ' . esc_html__($gftecustomSecs) . '</span>' . $gfteAddTimerText . '</p>';
		}
		$form_string = '
		<style type="text/css">
			.gftetimertext {
				position: relative !important;
				display: inline-block;
				width: 100%;
				margin: 0;
				padding: 0;
				font-family: helvetica;
				text-align: center !important;
				color: #000000;
				font-size: 16px;
				height: auto;
				top: 0px;
				clear: both;
			}
			.gftetimertext p {
				position: absolute;
				margin: 0;
				left: 50%;
				-webkit-transform: translateX(-50%);
  				-ms-transform: translateX(-50%);
				transform: translateX(-50%);
				word-break: keep-all;
			}
		</style>
		<div class="gftetimertext">
			' . $gfteClockType . '
		</div>' . $form_string;
	}
	if (isset($form['gftetimesupmessage']) && $form['gftetimesupmessage']) {
		$form_string = '<div id="gftetimesupmessage" style="display: none;">' . esc_html__($form['gftetimesupmessage']) . '</div>' . $form_string;
	}
    if (isset($form['gfteautoredirect']) && $form['gfteautoredirect']) {
        $form_string = '<div id="gfteautoredirect" style="display: none;">' . trim(esc_html__($form['gfteautoredirect'])) . '</div>' . $form_string;
    }
    if (isset($form['gfteclockendmessage']) && $form['gfteclockendmessage']) {
        $form_string = '<div id="gfteclockendmessage" style="display: none;">' . trim(esc_html__($form['gfteclockendmessage'])) . '</div>' . $form_string;
    }
    if (isset($form['gftetimedevents']) && $form['gftetimedevents']) {
        $filteredTimedEvents =  $form['gftetimedevents'];
        $filteredTimedEvents = apply_filters('gfteTimedEventsFilter', esc_html__($filteredTimedEvents));
        $form_string = '<div id="gftetimedevents" style="display: none;">' . trim(esc_html__($filteredTimedEvents)) . '</div>' . $form_string;   
    }
	// set cookies and check and perform single page access for those requiring a simple solution      
	//if(isset($_COOKIE['gfte_test'])) {
//		$form_string .= '<script type="text/javascript">
//			jQuery(function($){
//			"use strict";
//				$(document).ready(function(){
//					var alerted = false;
//					setInterval(function(){
//						if (alerted == false) {
//							alerted = true;
//							$("#gform_' . $form["id"] . '").hide();
//							$("#gform_' . $form["id"] . '").find(":input").not(":input[value=Submit], :input[value=Previous], :input[value=Next], :input[type=submit]").attr({"readonly": "readonly", "style": "pointer-events: none;"});
//							alert("You cant do this again");
//						} else {
//							window.close();
//						}	
//					}, 1000);
//					
//				});
//			});
//		</script>';
//	}
//	setcookie('gfte_test', 'working', time() + (86400 * 30), "/"); // 86400 = 1 day 

    return $form_string;
}



//ENTRY TIMER FIELD CLASS
function gfte_extension_class() {
	if(class_exists('GF_Field_Text')) {
         class GF_Field_Timer extends GF_Field_Text {
            public $type = 'gftetimer';

            public function get_form_editor_field_title() {
                return esc_attr__( 'Entry Timer', 'gravityforms' );
            }
            public function get_form_editor_button() {
                return array(
                    'group' => 'advanced_fields',
                    'text'  => $this->get_form_editor_field_title()
                );
            }
            function get_form_editor_field_settings() {
                return array(
                    'label_setting',
                    'label_placement_setting',
                    'admin_label_setting',
                    'size_setting',
                    'description_setting',
                    'css_class_setting',
                );
            }
        }
        GF_Fields::register(new GF_Field_Timer());

        class GF_Field_Page_Timer extends GF_Field_Text {
            public $type = 'gftepagetimer';

            public function get_form_editor_field_title() {
                return esc_attr__( 'Page Timer', 'gravityforms' );
            }
            public function get_form_editor_button() {
                return array(
                    'group' => 'advanced_fields',
                    'text'  => $this->get_form_editor_field_title()
                );
            }
            function get_form_editor_field_settings() {
                return array(
                    'label_setting',
                    'label_placement_setting',
                    'admin_label_setting',
                    'size_setting',
                    'description_setting',
                    'css_class_setting',
                );
            }
        }
        GF_Fields::register(new GF_Field_Page_Timer());
	}
}
add_action('plugins_loaded', 'gfte_extension_class');


/*GFORMS ENTRY TIMER FIELD OPTIONS*/
add_action('gform_field_standard_settings', 'gfte_timer_settings', 10, 2);
function gfte_timer_settings($position, $form_id) {
    //create settings on position 10 (right after Field Label)
	//$form = GFAPI::get_form($form_id);
    if ($position == 10) {
        ?>
		<!--timer fields-->
		<li class="gfte_timer_visibility_setting field_setting">
		       <label for="field_gfte_timer_visibility_value" class="section_label">
			<?php _e('Timer Settings', 'gravityforms');
				echo ' <a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank"><img src="' . plugin_dir_url(__FILE__)  . 'images/owl-small.png" height="13px" width="auto" style="display:inline-block;"></a>';
			?>
			</label>
			<input type="checkbox" id="field_gfte_timer_visibility_value" name="gftetimervisibilityOption" onClick="SetFieldProperty('gftetimervisibilityField', this.checked);" /> Timer Visibility <?php gform_tooltip('form_field_gfte_timer_visibility_value') ?>
		</li>
       	<li class="gfte_clock_type_setting field_setting">
			<input type="checkbox" id="field_gfte_clock_type_value" name="gfteclocktypeOption" onClick="SetFieldProperty('gfteclocktypeField', this.checked);" /> Countdown Timer <?php gform_tooltip('form_field_gfte_clock_type_value') ?>
		</li>
        <li class="gfte_hide_ontime_setting field_setting">
			<input type="checkbox" id="field_gfte_hide_ontime_value" name="gftehideontimeOption" onClick="SetFieldProperty('gftehideontimeField', this.checked);" /> Hide after Form Time Limit <?php gform_tooltip('form_field_gfte_hide_ontime_value') ?>
		</li>
		<li class="gfte_precise_clock_setting field_setting">
			<input type="checkbox" id="field_gfte_precise_clock_value" name="gftepreciseclockOption" onClick="SetFieldProperty('gftepreciseclockField', this.checked); " /> Precision Timer <?php gform_tooltip('form_field_gfte_precise_clock_value') ?>   <input type="number" id="field_gfte_precise_round_value" name="gftepreciseroundField" oninput="if (this.value > 0 && this.value < 5) {SetFieldProperty('gftepreciseroundField', this.value);} if (this.value <= 0){SetFieldProperty('gftepreciseroundField', 1);} if (this.value >= 5){SetFieldProperty('gftepreciseroundField', 4);}" min="1" max="4"> Fractional Seconds Precision <?php gform_tooltip('form_field_gfte_precise_round_value') ?> 
		</li>
        <li class="gfte_exclude_timer_setting field_setting">
			<input type="checkbox" id="field_gfte_exclude_timer_value" name="gfteexcludetimerOption" onClick="SetFieldProperty('gfteexcludetimerField', this.checked);" /> Exclude From Submission <?php gform_tooltip('form_field_gfte_exclude_timer_value') ?>
            <div style="width: 100%; height: 0px; border-bottom: 1px solid #00feff; margin-top: 12px;"></div>
		</li>
        <li class="gfte_hours_only_setting field_setting">
            <input type="radio" id="field_gfte_hours_only_value" name="gftetimedelimiterOption" onClick="SetFieldProperty('gftehoursonlyField', this.checked);SetFieldProperty('gfteminutesonlyField', this.unchecked);SetFieldProperty('gftesecondsonlyField', this.unchecked);" /> Save in Hours Only <?php gform_tooltip('form_field_gfte_hours_only_value') ?>
        </li>
        <li class="gfte_minutes_only_setting field_setting">
            <input type="radio" id="field_gfte_minutes_only_value" name="gftetimedelimiterOption" onClick="SetFieldProperty('gftehoursonlyField', this.unchecked);SetFieldProperty('gfteminutesonlyField', this.checked);SetFieldProperty('gftesecondsonlyField', this.unchecked);" /> Save in Minutes Only <?php gform_tooltip('form_field_gfte_minutes_only_value') ?>
        </li>
        <li class="gfte_seconds_only_setting field_setting">
            <input type="radio" id="field_gfte_seconds_only_value" name="gftetimedelimiterOption" onClick="SetFieldProperty('gftehoursonlyField', this.unchecked);SetFieldProperty('gfteminutesonlyField', this.unchecked);SetFieldProperty('gftesecondsonlyField', this.checked);" /> Save in Seconds Only <?php gform_tooltip('form_field_gfte_seconds_only_value') ?>
        </li>
        <li class="gfte_seconds_only_setting field_setting">
            <input type="radio" id="field_gfte_normal_save_value" name="gftetimedelimiterOption" onClick="SetFieldProperty('gftehoursonlyField', this.unchecked);SetFieldProperty('gfteminutesonlyField', this.unchecked);SetFieldProperty('gftesecondsonlyField', this.unchecked);" /> Save Normally
            <div style="width: 100%; height: 0px; border-bottom: 1px solid #00feff; margin-top: 12px;"></div>
        </li>
        <li class="gfte_remove_text_setting field_setting">
			<input type="checkbox" id="field_gfte_remove_text_value" name="gfteremovetextOption" onClick="SetFieldProperty('gfteremovetextField', this.checked);" /> Remove Time Text From Saved Value <?php gform_tooltip('form_field_gfte_remove_text_value') ?>
		</li>
		<br/>
        <?php
    }
}
/*GFORMS TIMED ENTRIES ADVANCED FIELD OPTIONS*/
add_action('gform_field_advanced_settings', 'gfte_timed_entry_settings', 10, 2);
function gfte_timed_entry_settings($position, $form_id) {

    //create settings on position 25 (right after Admin Field Label)
    if ($position == 50) {
        ?>
        <!--block editing setting for all fields-->
		<li class="gfte_timer_blocking_setting field_setting">
		       <label for="field_gfte_timer_blocking_value" class="section_label">
			<?php _e('Timed Entries', 'gravityforms');
				echo ' <a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank"><img src="' . plugin_dir_url(__FILE__)  . 'images/owl-small.png" height="13px" width="auto" style="display:inline-block;"></a>';
			?>
			</label>
			<input type="checkbox" id="field_gfte_timer_blocking_value" name="gftetimerblockingOption" onClick="SetFieldProperty('gftetimerblockingField', this.checked);" /> Keep Field Editable on Timed Blocked Editing. <?php gform_tooltip( 'form_field_gfte_timer_blocking_value' ) ?>
		</li>
		<br/>
        <?php
    }
}
//Add setting to fields of type
add_action('gform_editor_js', 'gfte_editor_script');
function gfte_editor_script(){
    ?>
    <script type='text/javascript'>
		fieldSettings["gftetimer"] += ", .gfte_timer_visibility_setting";
		fieldSettings["gftetimer"] += ", .gfte_clock_type_setting";
		fieldSettings["gftetimer"] += ", .gfte_hide_ontime_setting"; 
		fieldSettings["gftetimer"] += ", .gfte_precise_clock_setting";
		fieldSettings["gftetimer"] += ", .gfte_precise_round_setting";
        fieldSettings["gftetimer"] += ", .gfte_exclude_timer_setting";
        fieldSettings["gftetimer"] += ", .gfte_hours_only_setting";
        fieldSettings["gftetimer"] += ", .gfte_minutes_only_setting";
        fieldSettings["gftetimer"] += ", .gfte_seconds_only_setting";
        fieldSettings["gftetimer"] += ", .gfte_remove_text_setting";
		
		fieldSettings["gftepagetimer"] += ", .gfte_timer_visibility_setting";
		fieldSettings["gftepagetimer"] += ", .gfte_hide_ontime_setting"; 
		fieldSettings["gftepagetimer"] += ", .gfte_precise_clock_setting";
		fieldSettings["gftepagetimer"] += ", .gfte_precise_round_setting";
        fieldSettings["gftepagetimer"] += ", .gfte_exclude_timer_setting";
		fieldSettings["gftepagetimer"] += ", .gfte_hours_only_setting";
        fieldSettings["gftepagetimer"] += ", .gfte_minutes_only_setting";
        fieldSettings["gftepagetimer"] += ", .gfte_seconds_only_setting";
        fieldSettings["gftepagetimer"] += ", .gfte_remove_text_setting";
		
		fieldSettings["text"] += ", .gfte_timer_blocking_setting";
		fieldSettings["textarea"] += ", .gfte_timer_blocking_setting";
		fieldSettings["date"] += ", .gfte_timer_blocking_setting";
		fieldSettings["name"] += ", .gfte_timer_blocking_setting";
		fieldSettings["number"] += ", .gfte_timer_blocking_setting";
		fieldSettings["email"] += ", .gfte_timer_blocking_setting";
		fieldSettings["phone"] += ", .gfte_timer_blocking_setting";
		fieldSettings["quiz"] += ", .gfte_timer_blocking_setting";
		fieldSettings["time"] += ", .gfte_timer_blocking_setting";
		fieldSettings["website"] += ", .gfte_timer_blocking_setting";
		fieldSettings["address"] += ", .gfte_timer_blocking_setting";
		fieldSettings["select"] += ", .gfte_timer_blocking_setting";
		fieldSettings["radio"] += ", .gfte_timer_blocking_setting";
		fieldSettings["multiselect"] += ", .gfte_timer_blocking_setting";
		fieldSettings["checkbox"] += ", .gfte_timer_blocking_setting";
		fieldSettings["list"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_title"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_content"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_excerpt"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_category"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_tags"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_image"] += ", .gfte_timer_blocking_setting";
		fieldSettings["post_custom_field"] += ", .gfte_timer_blocking_setting";
		fieldSettings["fileupload"] += ", .gfte_timer_blocking_setting";

        //binding to the load field settings event to initialize options
        jQuery(document).bind("gform_load_field_settings", function(event, field, form){
			jQuery("#field_gfte_timer_visibility_value").attr("checked", field["gftetimervisibilityField"] == true);
			jQuery("#field_gfte_clock_type_value").attr("checked", field["gfteclocktypeField"] == true);
			jQuery("#field_gfte_hide_ontime_value").attr("checked", field["gftehideontimeField"] == true);
			jQuery("#field_gfte_precise_clock_value").attr("checked", field["gftepreciseclockField"] == true);
			jQuery("#field_gfte_precise_round_value").val(field["gftepreciseroundField"]);
            jQuery("#field_gfte_exclude_timer_value").attr("checked", field["gfteexcludetimerField"] == true);
			jQuery("#field_gfte_timer_blocking_value").attr("checked", field["gftetimerblockingField"] == true);
            jQuery("#field_gfte_hours_only_value").attr("checked", field["gftehoursonlyField"] == true);
            jQuery("#field_gfte_minutes_only_value").attr("checked", field["gfteminutesonlyField"] == true);
            jQuery("#field_gfte_seconds_only_value").attr("checked", field["gftesecondsonlyField"] == true);
            jQuery("#field_gfte_remove_text_value").attr("checked", field["gfteremovetextField"] == true);
        });
    </script>
    <?php
}
//GForm Field Settings Tooltips
add_filter('gform_tooltips', 'gfte_formtimer_tooltips');
function gfte_formtimer_tooltips($tooltips) {
	$tooltips['form_field_gfte_timer_visibility_value'] = "<h6>Timer Visibility</h6><p>Turn this on to make this timer visible to the user.</p>";
	$tooltips['form_field_gfte_clock_type_value'] 	= "<h6>Countdown Timer</h6><p>Turn this on to make this timer count down instead of up. This requires a time limit to be set in the forms settings.</p>";
    $tooltips['form_field_gfte_hide_ontime_value'] 	= "<h6>Hide after Form Time Limit</h6><p>Turn this on to hide this timer after the forms time limit has been reached.</p>";
	$tooltips['form_field_gfte_precise_clock_value'] 	= "<h6>Precise Clock</h6><p>Turn this on to make this timer measure fractional seconds.</p>";
	$tooltips['form_field_gfte_precise_round_value'] 	= "<h6>Fractional Seconds Precision</h6><p>Enter a number 1-4 to set the fractional seconds precision. Reported 'millseconds' (fractional decimal value of a second) are rounded to this number of digits.</p>";
	$tooltips['form_field_gfte_exclude_timer_value'] 	= "<h6>Exclude From Entry</h6><p>Turn this on to exclude this timers results from form submissions.</p>";
	$tooltips['form_field_gfte_timer_blocking_value'] = "<h6>Keep Field Editable</h6><p>Turn this on to make this field still editable by the user after editing has been blocked by the 'Gravity Forms timed Entries' 'Block Editing' option.</p>";
    $tooltips['form_field_gfte_hours_only_value'] = "<h6>Save in Hours Only</h6><p>Turn this on to make this field only save the time in hours to the nearest rounded hour.</p><p>This cancels Precision Timer if selected.</p>";
    $tooltips['form_field_gfte_minutes_only_value'] = "<h6>Save in Minutes Only</h6><p>Turn this on to make this field only save the time in minutes to the nearest rounded minute.</p><p>This cancels Precision Timer if selected.</p>";
    $tooltips['form_field_gfte_seconds_only_value'] = "<h6>Save in Seconds Only</h6><p>Turn this on to make this field only save the time in seconds to the nearest rounded second.</p><p>This can be used along with 'Precision Timer' if selected.</p>";
    $tooltips['form_field_gfte_remove_text_value'] = "<h6>Remove Time Text From Saved Value</h6><p>Turn this on to remove the time text ('hours, minutes, seconds'), and only save the time in integers.</p><p>This is only available if saving the time in hours, minutes, or seconds only, and can be useful if you need to process the saved field value without the text.</p>";
	return $tooltips;
}

/*GFORM TIMED ENTRIES PRE RENDER FILTER*/
add_filter('gform_pre_render', 'gfte_prerender', 10, 3);
function gfte_prerender($form, $ajax, $field_values) {
    $gfte_start_timer   = true;
    if (isset($form['gftetimerstartpage']) && $form['gftetimerstartpage']) {
        $gfte_current_page  = class_exists('GFFormDisplay') ? GFFormDisplay::get_current_page($form['id']) : false;
        $gfte_start_timer   = $gfte_current_page == $form['gftetimerstartpage'] ? true : false; 
    }
    if (isset($form['gftetimerstartpage']) && $form['gftetimerstartpage'] && !$gfte_start_timer) {
        return $form;
    }

	foreach($form['fields'] as &$field) {
		if ($field->type == 'gftetimer') {
			$field_post_value 		= rgpost('input_' . $field->id);
			if (!$field_post_value) {
				$startTime 				= current_time('Y/m/d H:i:s', new DateTimeZone("UTC"));
				$startTimeprecise		= microtime(true);
				$timelimit				= isset($form['gftetimelimit']) && $form['gftetimelimit'] ? $form['gftetimelimit'] : 'none';
                $timelimit              = apply_filters('gfteTimeLimitFilter', $timelimit);
                $startpage              = isset($form['gftetimerstartpage']) && $form['gftetimerstartpage'] ? $form['gftetimerstartpage'] : false;
                $synchinterval          = isset($form['gftetimesynchinterval']) && $form['gftetimesynchinterval'] ? $form['gftetimesynchinterval'] : false;
                $blockediting           = isset($form['gfteblockediting']) && $form['gfteblockediting'] ? $form['gfteblockediting'] : false;
                $autosubmit             = isset($form['gfteautosubmit']) && $form['gfteautosubmit'] ? $form['gfteautosubmit'] : false;
				$field->defaultValue 	= $startTime . '%' . $timelimit . '%' . $synchinterval . '%' . $form['id'] . '%' . $blockediting . '%' . $autosubmit . '%' . $startTimeprecise . '%' . $startpage  . '%CustomTime';
			} else {
                //UPDATE TO CURRENT FORMAT 
				//$gftetimerarray 	= explode('%', $field_post_value);
				//$gftebaseTime 		= strtotime(new DateTime($gftetimerarray[0]));
				//$gfteinitialtime 	= $gftetimerarray[0];
				//$gfteLockedForm		= $gftetimerarray[7] ? $gftetimerarray[7] : false;
				//$gftecurrentTime 	= strtotime(new DateTime(current_time('Y/m/d H:i:s')));//use above
				
				
				//$gfteTimeOffset		= $gftetimerarray[6] ? intval($gftetimerarray[6]) : floor($gftebaseTime - $gftecurrentTime);
				//$gfteTimeLimiter	= $gftetimerarray[1] === 'none' ? 'none' : intval($gftetimerarray[1]);
				//$gfteTimeSynch		= $gftetimerarray[2] === '' ? '' : (intval($gftetimerarray[2]) * 1000);
				//$gfteTimeSyninit	= $gftetimerarray[2] ? $gftetimerarray[2] : '';
				//$gfteFormId			= $gftetimerarray[3] ? $gftetimerarray[3] : '';
				//$gfteBlockEdit		= $gftetimerarray[4] ? $gftetimerarray[4] : '';
				//$gfteAutoSubmit		= $gftetimerarray[5] ? $gftetimerarray[5] : '';
				
				
				//if (($gftecurrentTime - $gftebaseTime) > $form['gftetimelimit'] || $gfteLockedForm) { //decide on hiding or readonly and apply pre render
					//foreach($form['fields'] as &$field) {
						//if (strpos($field->cssClass, 'gftehidden') === false) {
//							$field->cssClass .= ' gftereadonly gftehidden';
//						}
					//}
				//}
			} 
		}
		if ($field->type == 'gftepagetimer' && !$ajax) {
			$field->adminOnly = true;
		}
	}
	return $form;
}

/*GFORM ENTRY TIMER FIELD FRONT END DISPLAY*/
add_filter( 'gform_field_content', 'gfte_subsection_field', 10, 5 );
function gfte_subsection_field($content, $field, $value, $lead_id, $form_id) {
    if (($field->type == 'gftetimer' || $field->type == 'gftepagetimer') && $field->gftetimervisibilityField == true && (!$field->is_form_editor() && !$field->is_entry_detail() && !$field->is_entry_detail_edit())) {
		$content = str_replace( "type=", "style='display: none;' type=", $content );
        $form = GFAPI::get_form($form_id);
        $gftecustomHours    = isset($form['gftecustomtimeseparatorhours']) && $form['gftecustomtimeseparatorhours'] ? $form['gftecustomtimeseparatorhours'] : "Hours";
        $gftecustomMins     = isset($form['gftecustomtimeseparatormins']) && $form['gftecustomtimeseparatormins'] ? $form['gftecustomtimeseparatormins'] : "Minutes";
        $gftecustomSecs     = isset($form['gftecustomtimeseparatorsecs']) && $form['gftecustomtimeseparatorsecs'] ? $form['gftecustomtimeseparatorsecs'] : "Seconds";
		
		
		$gftefieldClockType	= '<p class="gfteCountupTimer"><span class="gftehours">0</span><span class="gftehoursseparator"> ' . esc_html__($gftecustomHours) . ' </span><span class="gfteminutes">0</span><span class="gfteminsseparator"> ' . esc_html__($gftecustomMins) . ' </span><span class="gfteseconds">0</span><span class="gftesecondssseparator"> ' . esc_html__($gftecustomSecs) . '</span></p>';
		
		if ($field->gfteclocktypeField == true) {
			$gftefieldClockType	= '<p class="gfteCountdownTimer"><span class="gfte-hours">0</span><span class="gfte-hoursseparator"> ' . esc_html__($gftecustomHours) . ' </span><span class="gfte-minutes">0</span><span class="gfte-minsseparator"> ' . esc_html__($gftecustomMins) . ' </span><span class="gfte-seconds">0</span><span class="gfte-secondssseparator"> ' . esc_html__($gftecustomSecs) . '</span></p>';
		}
		
		if ($field->type == 'gftepagetimer') {
            $field_post_value   = rgpost('input_' . $field->id);
            $startTime 			= current_time('Y/m/d H:i:s', new DateTimeZone("UTC"));
            $startTimePrecise	= microtime(true);
            $pageToTime         = $field->pageNumber;
            $currentPage        = GFFormDisplay::get_current_page($form['id']);
            $pageTimerRun       = $pageToTime == $currentPage ? 'active' : 'pause';
            
            if (!$field_post_value) {
                $content                = str_replace('value', 'value="' . $startTimePrecise . '%' . '0' . '%' . $pageTimerRun . '"', $content);
            } else {
                $endprecise 	= microtime(true);
                $field_value 	= explode('%', $field_post_value);
                $preciseTime	= $field_value[0];
                $pageTime       = $field_value[1];
                $pageRecord     = $field_value[2];
                if ($pageRecord =='active') {
                    $pageTime   = ($endprecise - $preciseTime) + $pageTime;       
                }
                $content    = preg_replace('/value([^"]+)"/', 'value="' . $startTimePrecise . '%' . $pageTime . '%' . $pageTimerRun . '"', $content, 1);
            }
			
			$gftefieldClockType	= '<p class="gftePageTimerDisplay"><span class="gftepagehours">0</span><span class="gftepagehoursseparator"> ' . esc_html__($gftecustomHours) . ' </span><span class="gftepageminutes">0</span><span class="gftepageminsseparator"> ' . esc_html__($gftecustomMins) . ' </span><span class="gftepageseconds">0</span><span class="gftepagesecondssseparator"> ' . esc_html__($gftecustomSecs) . '</span></p>';
        }
		
        $gfteextraClasses = '';
        if ($field->gftehideontimeField) {
            $gfteextraClasses .= ' gfte-hide';
        }
		if ($field->type == 'gftepagetimer') {
            $gfteextraClasses .= ' gftePageTimerText_' . $field->pageNumber;
        }
		
		$content = $content . '</li>
		<style type="text/css">
			.gftetimertext {
				position: relative !important;
				display: inline-block;
				width: 100%;
				margin: 0;
				padding: 0;
				font-family: helvetica;
				text-align: center !important;
				color: #000000;
				font-size: 16px;
				height: 14px;
				top: 0px;
				clear: both;
			}
			.gftetimertext p {
				position: absolute;
				margin: 0;
				left: 50%;
				-webkit-transform: translateX(-50%);
  				-ms-transform: translateX(-50%);
				transform: translateX(-50%);
				word-break: keep-all;
			}
		</style>
		<div class="gftetimertext' . $gfteextraClasses . '">
			' . $gftefieldClockType . '
		</div>';
    } else if (($field->type == 'gftetimer' || $field->type == 'gftepagetimer') && $field->gftetimervisibilityField == false && (!$field->is_form_editor() && !$field->is_entry_detail() && !$field->is_entry_detail_edit())){
		$content = $content . '</li>
		<style type="text/css">
			.gftehiddentimer {
				display: none !important;
			}
		</style>';
	}
    return $content;
}

/*GFORM TIMED ENTRIES VALIDATE ALL DATA LOGIC*/
add_filter('gform_validation', 'gfte_validate');
function gfte_validate($validation_result) {
	$form = $validation_result['form'];
	if ($form['gftetimelimit'] && $form['gfteautosubmit']) {
		foreach($form['fields'] as &$field) {	
			$validation_result['is_valid'] 	= true;
			$field->failed_validation 		= false;
		}
		$validation_result['form'] = $form;
		return $validation_result;
	} else {
		return $validation_result;
	}
}

/*GFORM TIMED ENTRIES SAVED TIMER FIELD FORMATTING*/
add_filter('gform_save_field_value', 'gfte_timed_entries_field_value', 1, 4);
function gfte_timed_entries_field_value($value, $lead, $field, $form) {
	if (isset($field->type)  && ($field->type == 'gftetimer' || $field->type == 'gftepagetimer') && $value && !get_option('gfe_encrypt_decrypt')) {
		
		if ($field->gfteexcludetimerField) {
			return null;
		}
		
		$endprecise 	= microtime(true);
        $endTime		= current_time('Y/m/d H:i:s', new DateTimeZone("UTC"));
		$preciseTime	= false;
        $pageTime       = false;
		$hourstext  	= $form['gftecustomtimeseparatorhours'] ?  ' ' . $form['gftecustomtimeseparatorhours'] . ' ' : ' hours ';
        $minstext   	= $form['gftecustomtimeseparatormins'] ?  ' ' . $form['gftecustomtimeseparatormins'] . ' ' : ' minutes ';
        $secstext   	= $form['gftecustomtimeseparatorsecs'] ?  ' ' . $form['gftecustomtimeseparatorsecs'] . ' ' : ' seconds ';
		
		if ($field->gfteremovetextField && ($field->gftehoursonlyField || $field->gfteminutesonlyField || $field->gftesecondsonlyField)) {
            $hourstext  = '';
            $minstext   = '';
            $secstext   = '';
        }
		
		$value = explode('%', $value);
		
		if ($field->type == 'gftetimer') {
          $value 		= isset($value[0]) ? $value[0] : $value;
          $precise		= isset($value[7]) ? intval($value[7]) : 0;

          //interim replace custom timer text
          $value = $form['gftecustomtimeseparatorhours'] ? str_replace($form['gftecustomtimeseparatorhours'], 'hours', $value) : $value;
          $value = $form['gftecustomtimeseparatormins'] ? str_replace($form['gftecustomtimeseparatormins'], 'minutes', $value) : $value;
          $value = $form['gftecustomtimeseparatorsecs'] ? str_replace($form['gftecustomtimeseparatorsecs'], 'seconds', $value) : $value;

          if (strpos($value, '.')) {
              $value 	= explode('.', $value);
              $value 	= $value[0] . ' seconds';
          }
          if ($field->gfteremovetextField && ($field->gftehoursonlyField || $field->gfteminutesonlyField || $field->gftesecondsonlyField)) {
              try {
                  $date = new DateTime($value);
              } catch (Exception $e) {
                  if ($field->gftehoursonlyField) {
                      $value = trim($value) . ' hours';
                  }
                  if ($field->gfteminutesonlyField) {
                      $value = trim($value) . ' minutes';
                  }
                  if ($field->gftesecondsonlyField && !strstr($value, 'seconds')) {
                      $value = trim($value) . ' seconds';
                  }
              }
          }

          $startTime 	= new Datetime($value);
          $endTime 		= new Datetime($endTime);
          $formTime 	= $startTime->diff($endTime);
          $days 		= $formTime->format("%a");
          $hours 		= $formTime->format("%H");
          $minutes 		= $formTime->format("%I");
          $seconds 		= $formTime->format("%S");
          $milliseconds = '';
          $totalseconds = ($days * 24 * 3600) + ($hours * 3600) + ($minutes * 60) + $seconds;

          if ($field->gftepreciseclockField) {
              $formTimeX  = ($endprecise - $precise);
              $roundTimeX = $field->gftepreciseroundField ? intval($field->gftepreciseroundField) : false;
              if ($field->gftepreciseroundField) {
                  $formTimeX = round($formTimeX, $field->gftepreciseroundField);
              } 
              $milliseconds 	= explode('.', $formTimeX);
              $milliseconds 	= isset($milliseconds[1]) ? '.' . $milliseconds[1]: '';
          }
			
          $days 	= $days == 0 ? '' : ltrim($days, '0') . ' days ';
          $hours 	= $hours == 0 ? '' : ltrim($hours, '0') . $hourstext;
          $minutes 	= $minutes == 0 ? '' : ltrim($minutes, '0') . $minstext;
          $seconds 	= $seconds == 0 ? '' : ltrim($seconds, '0') . $milliseconds . $secstext;
          $value	= '' . $days . $hours . $minutes . $seconds . '';
          $field->gftehoursonlyField ? $value = floor($totalseconds / 36000) . $hourstext: false;
          $field->gfteminutesonlyField ? $value = floor($totalseconds / 60) . $minstext: false;
          $field->gftesecondsonlyField ? $value = $totalseconds . $milliseconds . $secstext: false;
			
          return $value;
			
		} else if ($field->type == 'gftepagetimer') {
            $preciseTime	= isset($value[0]) ? $value[0] : false;
            $pageTime       = isset($value[1]) ? $value[1] : false;
            $pageRecord     = isset($value[2]) ? $value[2] : false;
            if ($pageRecord =='active') {
                $pageTime   = ($endprecise - $preciseTime) + $pageTime; 
            }
			$pageTimeVal 	= explode('.', $pageTime);
			$pageMilsecs    = isset($pageTimeVal[1]) ? intval($pageTimeVal[1]) : false;
			$pageTimeInt    = isset($pageTimeVal[0]) ? intval($pageTimeVal[0]) : $pageTime;
			$days           = floor($pageTimeInt / 86400);
			$pageTimeInt    = $pageTimeInt % 86400;
			$hours          = floor($pageTimeInt / 3600);
			$pageTimeInt    = $pageTimeInt % 3600;
			$minutes        = floor($pageTimeInt / 60);
			$seconds        = $pageTimeInt % 60;
			if ($field->gftepreciseclockField && $pageMilsecs) {
				$roundTimeX = $field->gftepreciseroundField ? intval($field->gftepreciseroundField) : false;
				if ($roundTimeX) {
					$pageMilsecs = round($pageTime, $roundTimeX);
				}
				$pageMilsecs = explode('.', $pageMilsecs);
				$pageMilsecs = isset($pageMilsecs[1]) ? '.' . $pageMilsecs[1] : '';
			} else {
				$pageMilsecs = '';
			}
			$days 			= $days == 0 ? '' : ltrim($days, '0') . ' days ';
			$hours 			= $hours == 0 ? '' : ltrim($hours, '0') . ' ' . $hourstext . ' ';
			$minutes 		= $minutes == 0 ? '' : ltrim($minutes, '0') . ' ' . $minstext . ' ';
			$seconds 		= $seconds == 0 ? '' : ltrim($seconds, '0') . $pageMilsecs . ' ' . $secstext;
			$value			= '' . $days . $hours . $minutes . $seconds . '';
			$totalseconds 	= $seconds + ($minutes * 60) + ($hours * 3600) + ($days * 86400);
			$field->gftehoursonlyField ? $value = floor($totalseconds / 36000) . $hourstext: false;
          	$field->gfteminutesonlyField ? $value = floor($totalseconds / 60) . $minstext: false;
          	$field->gftesecondsonlyField ? $value = $totalseconds . ' ' . $secstext: false;

			return $value;
        } 
	} else {
		return $value;
	}
}

/*GFORMS TIMED ENTRIES FIELD CLASSES*/
add_filter('gform_field_css_class', 'gfte_custom_classes', 10, 3);
function gfte_custom_classes($classes, $field, $form) {
	/*TIMER*/
	if ($field->type == 'gftetimer') {
        $classes .= ' gftereadonly gfteStartTime';
    }
	if ($field->type == 'gftepagetimer') {
        $classes .= ' gftereadonly gftePageTime gftePageNumber_' . $field->pageNumber;
    }
	if (($field->type == 'gftetimer' || $field->type == 'gftepagetimer') && $field->gftetimervisibilityField !== true) {
        $classes .= ' gftehiddentimer';
    }
    if (($field->type == 'gftetimer' || $field->type == 'gftepagetimer') && $field->gftehideontimeField === true) {
        $classes .= ' gfte-hide';
    }
	if ($field->gftetimedreadonly == true) {
        $classes .= ' gftetimedreadonly';
    }
	if ($field->gftetimerblockingField == true) {
		$classes .= ' gfteNoBlock';
	}
    return $classes;
}

//CREATE SETTINGS PAGE MENU
add_action('admin_menu', 'gfte_gravity_forms_timed_entries_menu');
function gfte_gravity_forms_timed_entries_menu() {
	//create new top-level menu
	add_options_page('Gravity Forms Timed Entries Settings', 'GF Timed Entries', 'edit_users', 'gravity-forms-timed-entries', 'gravity_forms_timed_entries_settings_page');

	//call register settings function
	add_action( 'admin_init', 'register_gravity_forms_timed_entries_settings' );
}

function register_gravity_forms_timed_entries_settings() {
	//register our settings
	register_setting('gravity_forms_timed_entries_settings-group', 'gfte_delete_options');
}

function gravity_forms_timed_entries_settings_page() {
	function gfte_get_plugin_version() {
		$plugin_data = get_plugin_data(__FILE__);
		$plugin_version = $plugin_data['Version'];
		return $plugin_version;
	}
	function gfte_check_php() {
        if (version_compare( phpversion(), '5.6.0', '>=')) {
            echo '<img src="' . plugin_dir_url(__FILE__) . 'images/check.png" height="14" width="14"> ' . esc_html__('PHP Version Supported!');
        } else {
            echo '<img src="' . plugin_dir_url(__FILE__) . 'images/x.png" height="12" width="12"> ' . esc_html__('PHP Version Not Supported! Requires 5.6.0+');
        }
    }
    function gfte_check_wp_version() {
        if (version_compare(get_bloginfo('version'), '4.6.1', '>=')) {
            echo '<img src="' . plugin_dir_url(__FILE__) . 'images/check.png" height="14" width="14"> ' . esc_html__('Wordpress Version Supported!');
        } else {
            echo '<img src="' . plugin_dir_url(__FILE__) . 'images/x.png" height="12" width="12"> ' . esc_html__('Wordpress Version Not Supported! Requires 4.6.1+');
        }
    }
    function gfte_check_gf_version() {
        if (version_compare(GFCommon::$version, '2.0.7', '>=')) {
            echo '<img src="' . plugin_dir_url(__FILE__) . 'images/check.png" height="14" width="14"> ' . esc_html__('Gravity Forms Version Supported!');
        } else {
            echo '<img src="' . plugin_dir_url(__FILE__) . 'images/x.png" height="12" width="12"> ' . esc_html__('Gravity Forms Version Not Supported! Requires 2.0.7+');
        }
    }
    ?>
    <style>
		input[type="text"], 
		textarea {
			border-radius: 7px;
			border: 2px;
		}
		input[type="text"] {
			width:100%;
			border: 2px solid #ddd;
		}
		button {
			border: 2px solid #00feff;
			background: none;
			background-color: #ffffff;
			border-radius: 7px;
			padding: 5px;
		}
		textarea.redtextarea {
			width: 100%;
			border: 2px solid red;
			border-radius: 7px;
		}
		input.redtext {
			width: 100%;			
			border: 2px solid red;
			border-radius: 7px;
		}
		table th.settings {
			background-color:#FFFFFF;
			padding: 10px;
			text-align: center;
		}
		table tr.settingstr {
			border-top: 2px solid #00feff;
		}
		table tr.settingstrgrey {
			border-top: 2px solid #ddd;
		}
		table.form-table {
			max-width: 830px;
		}
		.center {
			 text-align: center;
		}
		table.gftesystemcheck tr:nth-child(even) {
			background-color: #eeeeee;
		}
	</style>
    <div class="wrap gfte-admin-page" style="position:relative;">
    <?php echo '<img src="' . plugin_dir_url(__FILE__)  . 'images/GFTE-icon80x80.jpg" height="80" width="80" style="display:inline-block; float:left; margin-right: 10px;">'; ?>
    <h1 style="display:inline;">Gravity Forms Timed Entries</h1><span> ver <?php echo gfte_get_plugin_version(); ?></span>
    <p style="margin-top:0px;">by <a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank">Plugin Owl</a></p>
	<p style="margin-top:0px;"><a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank">Check out our other plugins!</a></p><br/>
    <form method="post" action="options.php" autocomplete="invalid">
        <?php settings_fields('gravity_forms_timed_entries_settings-group'); ?>
        <?php do_settings_sections('gravity_forms_timed_entries_settings-group'); ?>
    	<div class="gfte-sticky-save" style="position:fixed; top:20px; right:5px;"><?php submit_button();?></div>
        <table class="form-table">
           <tr class="settingstr" valign="top">
                <th class="settings" scope="row">SYSTEM CHECK</th>
                <td>
                    <div id="gfte-system-check" style="max-width:600px;">
                        <table class="gftesystemcheck"
                        style="
                        width: 100%;
                        max-width:600px;
                        background-color: #FFFFFF;
                        border: 2px solid #666666;
                        border-radius: 7px;
                        padding: 0px;
                        margin: 0;
                        font-size: 16px;
                        line-height: 30px">
                            <tr valign="top">
                                <td>PHP Version:</td>
                                <td><strong><?php echo phpversion(); ?></strong></td>
                                <td><?php echo gfte_check_php(); ?></td>
                            </tr>
                            <tr valign="top">
                                <td>Wordpress Version:</td>
                                <td><strong><?php echo get_bloginfo( 'version' ); ?></strong></td>
                                <td><?php echo gfte_check_wp_version(); ?></td>
                            </tr>
                            <tr valign="top">
                                <td>Gravity Forms Version:</td>
                                <td><strong><?php echo GFCommon::$version; ?></strong></td>
                                <td><?php echo gfte_check_gf_version(); ?></td>
                            </tr>
                        </table>
                    </div>
                    <br/>
				</td>
            </tr>
			<tr class="settingstr" valign="top">
				<th class="settings" scope="row">INSTRUCTIONS</th>
				<td>
                    <p>Make sure system meets above system check requirements, read usage instructions and configure options below.</p>
                    <br/>
					<br/>
					<p><strong>GETTING STARTED</strong><br/>
					To begin timing form submissions, add an entry timer field to the form. Entry Timers are added from the "advanced fields" in the Gravity Forms form editor.</p>
					<p>Note: If any changes you have made do not seem to take place, please clear any page and/or browser cache and reload the form page.</p>
					<br/>
					<p>The entry timer field will automatically record the total amount of time taken from initial page load of the form to entry submission and save this total time to this fields data within the entry. To the user the entry timer fields are not editable or visible, but optionally show up as a clock which shows the current total time either taken or remaining to fill out the form.</p>
					<br/>
					<p>Multiple timers can be added to a single form if you wish to display the current total time used or remaining time to the user in specific places throughout the form or display the same total time taken at specific places in the entry results.</p>
					<br/>
					<br/>
					<p><strong>ENTRY TIMER FIELD OPTIONS</strong><br/>Each timer field added to a form has its own set of options which are configured in the fields "Gereral" settings tab in the "Timer Settings" section within the form editor. Most settings are also available for individual page timers.</p>
					<br/>
					<p><b>Timer Visibility:</b> Sets individual entry timers visibility to the user.</p>
					<br/>
					<p><b>Countdown Timer:</b> Switches entry timer fields to count down instead of up. In order to use this option for any timer, a form time limit must be set in the forms time limit option to give the timers a time to count down from.</p>
                    <br/>
                    <p><b>Hide after Form Time Limit:</b> Hides the timer when the form time limit is reached if a form time limit is set.</p>
					<br/>
					<p><b>Precision Timer:</b> Makes the timer field record fractions of a second. The "Fractional Seconds Precision" modifier sets the number of decimal places to round to (1-4).</p>
                    <br/>
					<p><b>Exclude From Submission:</b> Allows for entry timer fields to have their time recording results excluded from the submission. This is useful if you have multiple timers visible to the user but only need one or none to actually record the time taken for the entry.</p>
					<br/>
					<p><b>Save in Hours Only:</b> Allows for entry timer fields to have their time recording results excluded from the submission. This is useful if you have multiple timers visible to the user but only need one or none to actually record the time taken for the entry.</p>
					<br/>
					<p><b>Save in Minutes Only:</b> Allows for entry timer fields to have their time recording results excluded from the submission. This is useful if you have multiple timers visible to the user but only need one or none to actually record the time taken for the entry.</p>
					<br/>
					<p><b>Save in Seconds Only:</b> Allows for entry timer fields to have their time recording results excluded from the submission. This is useful if you have multiple timers visible to the user but only need one or none to actually record the time taken for the entry.</p>
					<br/>
					<p><b>Remove Time Text From Saved Value:</b> Allows for entry timer fields to have their time recording results excluded from the submission. This is useful if you have multiple timers visible to the user but only need one or none to actually record the time taken for the entry.</p>
					<br/>
					<br/>
					<p><strong>PAGE TIMERS</strong><br/>
					<p>Page Timers can only be used when an Entry Timer has been added to the form, and are added from the "Advanced Fields" in the form editor. Page Timers are currently only allowed on <br/><a href="https://docs.gravityforms.com/shortcodes/#example-form-shortcode" target="blank" rel="nofollow noopener">AJAX enabled forms</a>.<br/><br/>
					<b>To use a page timer:</b><br/>
						1. Use paging in your form.<br/>
						2. Insert one or more Page Timers somewhere in the page you would like to time. <br/><br/>
						Page Timers may not be 100% visually accurate for the time displayed as they currently rely on the users browser for display, but they will always accurately record the total amount of time the user spends on the speicific form page it is placed in, and save the accurate time in the entry. Most of the field options for Entry Timers are also available for Page Timers in the form editor.</p>
					<br/>
					<br/>
                    <p><strong>FORM TIMED ENTRY OPTIONS</strong><br/>
					The form time limit options (time limit/auto submit/block editing/server synch interval) can only be used after an entry timer has been added to the form, and are then accessed through the forms settings page in a section labled "Form Timed Entries'.</p>
					<br/>
                    <p><b>Show Main Timer:</b> Displays a main form entry timer on the top of the form for each page.</p>
					<br/>
					<p><b>Countdown Timer:</b> Sets the main form timer to count down from the time limit instead of counting up. A time limit must be set on the form for this to become active.</p>
					<br/>
                    <p><b>Clock End Message:</b> Sets custom text to display in place of ALL entry timer countdown fields and the main form countdown timer when the time limit is reached. Default: Time Limit Reached. This requires a time limit to be set on the form.</p>
					<br/>
					<p><b>Time Limit:</b> Allows for the form to have a time limit which is set in the number of hours, minutes, and seconds in this option. Delete the option entry and update the form to remove any time limit.</p>
					<br/>
					<p><b>Synch Interval:</b> Sets how often (set in seconds) the timer should check the server for an accurate time to avoid users getting around the local timer by changing their system clock. The MINIMUM time is 5 seconds.<br/>
					NOTICE: You should consider the potential load on your server for how often you run this check and how many people could be using the form at a time.</p>
					<br/>
                    <p><b>Timing Start Page:</b> Hides All form timers and delays the start of timing until this form page number is visited.</p>
					<br/>
					<p><b>Block Editing*:</b>  Disallow further editing of the form once the time limit has been reached. A time limit must be set on the form for this to become active. ** &nbsp Individual fields can be excluded from the blocked editing by using the 'Timed Entry' options in thier advanced tab within the form editor.</p>
					<br/>
					<p><b>Time is Up Message:</b> Enter a custom message to display to the user with an alert when the form time limit has been reached.</p>
					<br/>
					<p><b>Auto Submit*:</b> Automatially submit the form once the time limit has been reached. A time limit must be set on the form for this to become active. **</p>
					<br/>
                    <p>* The "Block Editing" and "Auto Submit" functionality of this plugin are intended and best suited for single page ajax enabled forms/quizes with no required fields where users cannot enter invalid data into fields that would block normal submission or could be malicious. However, the functionality of these options will work on multi page non ajax enabled forms as well with subtle differences (refer to these options inline documentaion and tooltips for information on multip page/non-ajax useage).</p>
					<br/>
					<p>** It is recommended that you give the user details on what the time limit is and what will happen when the time is out using the "Time is Up Message" alert setting on the form settings page, an HTML field, the forms description, elsewhere on the page the form is displayed on, or prior to accessing the form. Read their option information on the forms settings page for further details.</p>
                    <br/>
                    <p><b>Custom Time Text:</b> Customize the 'Hours', 'Minutes', and 'Seconds' text for all timers.</p>
					<br/>
                    <p><b>Additional Timed Events:</b> Set additional events to happen at specified times. You can block editing, show, or hide a given CSS class; or alert a custom message.</p><p>Enter each event on a <b>new line</b> in the option.</p><p>The events are entered in a certain format. ie: event type::CSS class or message::seconds</p><p>Available event types are hide, show, block, alert.</p><p>EXAMPLE: show::myCssClass::14<br/>This will display any elements with the "MyCssClass" CSS class 14 seconds after the form timers are started.</p><p>EXAMPLE: block::myCssClass::120<br/>This will block editing of any form fields given the CSS class "MyCssClass" 120 seconds after the form timers are started.</p><p>EXAMPLE: alert::My custom alert!::60<br/>This will alert "My custom alert!" 60 seconds after the form timers are started.</p><p>NOTE: if you are using an Additional Timed Event to show a hidden form field you should use a CSS class to hide the form field at the start instead of making it a hidden field. You can use the class "gfte-hidden" to hide form fields until you want to display them with an Additional Timed Event. CSSclasses can be added to a field from the fields "advanced tab" in the form editor.</p>
					<br/>
                    <br/>
					<p><strong>TIMED VISIBILITY OPTIONS</strong><br/>
					<p>Page elements or form fields can be displayed or hidden when the form time limit expires. This requires a form time limit to be set. Fields to show on time limit should have thier visibility set to "visible". To add a CSS class to a field use the fields "Appearance" tab and enter the class in the "Custom CSS Class". Mulitple classes should be separated by a space.</p>
					<p><b>Show after the form time limit:</b> use the CSS class gfte-show.</p>
					<p><b>Hide after the form time limit:</b> use the CSS class gfte-hide.</p>
					<br/>
                    <p>Instructions for any individual options are written inline next to the option with most important information listed in the options tooltip. <b>It is strongly reccommended to read all inline and tooltip information on any option before using it.</b></p>
					<br/>
                    <br/>
                    <p><strong>UPDATING THE PLUGIN</strong><br/>
					<p>Please use the free <a href="https://envato.com/market-plugin/" target="blank" rel="nofollow noopener">Envato Market</a> plugin to recieve update notifications, and automatic updates for the plugin. Or you can follow the manual upgrade instructions within this plugins readme file.</p>
                    <br/>
                    <br/>
                    <p><strong>CSS CLASSES FOR CUSTOM STYLING</strong><br/>
                    <p>Count Up Timers:</p>
                    <p>outer div class = "gftetimertext"</p> 
                    <p>inner div class = "gfteCountupTimer"</p>
                    <p>hours display class = "gftehours"</p>
                    <p>hours text class = "gftehoursseparator"</p>
                    <p>minutes display class = "gfteminutes"</p>
                    <p>minutes text class = "gfteminsseparator"</p>
                    <p>seconds display class = "gfteseconds"</p>
                    <p>seconds text class = "gftesecondssseparator"</p>
                    <br/>
                    <p>Count Down Timers:</p>
                    <p>outer div class = "gfte-timertext"</p> 
                    <p>inner div class = "gfteCountdownTimer"</p>
                    <p>hours display class = "gfte-hours"</p>
                    <p>hours text class = "gfte-hoursseparator"</p>
                    <p>minutes display class = "gfte-minutes"</p>
                    <p>minutes text class = "gfte-minsseparator"</p>
                    <p>seconds display class = "gfte-seconds"</p>
                    <p>seconds text class = "gfte-secondssseparator"</p>
                    <br/>
					<p>Page Timers:</p>
                    <p>outer div class = "gftePageTimerText_(the page number)"</p> 
                    <p>inner p class = "gftePageTimerDisplay"</p>
                    <p>hours display class = "gftepagehours"</p>
                    <p>hours text class = "gftehourspageseparator"</p>
                    <p>minutes display class = "gftepageminutes"</p>
                    <p>minutes text class = "gftepageminsseparator"</p>
                    <p>seconds display class = "gftepageseconds"</p>
                    <p>seconds text class = "gftepagesecondssseparator"</p>
					<br/>
                    <p>You can easily target all Entry Timers for additional styling by adding custom CSS to your website similar to the following:<br/>
                    .gfteCountupTimer,<br/> 
                    .gfteCountdownTimer {<br/>
                       font-size: 24px;<br/>
                       color: aqua;<br/>
                       font-family: Gotham, "Helvetica Neue", Helvetica, Arial, "sans-serif";<br/>
                    }</p>
                    <br/>
                    <br/>
					<p><strong>PLUGIN FILTERS (for developers)</strong><br/>
                    <p><b>Filter: 'gftetimeLimitFilter'</b><br/>
                        Filters the main set time limit of the form when it is sent to the page on timer start. Return the new time limit using a number of total seconds.
                    </p>
                    <p>EXAMPLE:<br/>
                    function gftefilterexample($timelimit) {<br/>
                        return $timelimit * 2; //multiplies the time limit X 2 -returns 4 if time limit is 2<br/>
                    }<br/>
                    add_filter('gfteTimeLimitFilter', 'gftefilterexample');
                    </p>
                    <br/>
                    <br/>
                    <p><b>Filter: 'gfteTimedEventsFilter'</b><br/>
                        Filters the variable timed events option when sent to the page. These should be parsed, changed, and then reformatted for return as they are in the normal option with each event on a new line. 
                    </p>
                    <p>EXAMPLE:<br/>
                    function gftefilterexample($timedEvents) {<br/>
                        return 'alert::1 Minute!::60<br/>alert::2 Minutes!::210';<br/>
                    }<br/>
                    add_filter('gfteTimedEventsFilter', 'gftefilterexample');
                    </p>
                </td>
            </tr>
            <tr class="settingstr" valign="top">
                <th class="settings" scope="row">Save Plugin Options on Uninstall</th>
                <td>
				    <input type="checkbox" name="gfte_delete_options" value="1" <?php checked(1, get_option('gfte_delete_options'), true); ?> />
					<div id="gfte-delete-options-instructions" style="max-width:600px;">
						<p>Turn this on to save the current plugin options on uninstallation.</p><br/>
					</div>
                </td>
            </tr>
        </table>  
        <?php submit_button();?>        
    </form>
    <?php echo '<img src="' . plugin_dir_url(__FILE__)  . 'images/owl-small.png" height="30" width="30" style="display:inline-block; float:left;">'; ?><p> <a href="https://codecanyon.net/user/pluginowl/portfolio" target="_blank">Plugin Owl</a></p>
    </div>
<?php }

?>