<?php

/*
 * Class Name : GFHANNANSMS_Pro_{strtoupper( php file name) }	
 */

class GFHANNANSMS_Pro_CEQUENS {

	/*
	* Gateway title	
	*/
	public static function name( $gateways ) {

		$name = __( 'Cequens', 'GF_SMS' );

		$gateway = array( strtolower( str_replace( 'GFHANNANSMS_Pro_', '', get_called_class() ) ) => $name );

		return array_unique( array_merge( $gateways, $gateway ) );
	}


	/*
	* Gateway parameters
	*/
	public static function options() {
		return array(
			'apiKey'   => __( 'Api Key', 'GF_SMS' ),
			'userName' => __( 'User Name', 'GF_SMS' ),
		);
	}

	/*
	* Gateway credit
	*/
	public static function credit() {
		return true;
	}

	/*
	* Gateway action
	*/
	public static function process( $options, $action, $from, $to, $messages ) {

		if ( $action == 'credit' && ! self::credit() ) {
			return false;
		}
		if ( ! extension_loaded( 'curl' ) ) {
			return __( 'Please enable cURL extension in PHP', 'GF_SMS' );
		}

		$apiKey   = $options['apiKey'];
		$userName = $options['userName'];


		$ch = curl_init( 'http://api.cequens.com/cequens/api/v1/signin' );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode( array( 'apiKey' => $apiKey, 'userName' => $userName ) ) );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, array( 'Content-Type:application/json' ) );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		$response = curl_exec( $ch );
		curl_close( $ch );
		if ( empty( $response ) ) {
			return 'SignIn ::: ' . __( 'Unknown Error.', 'GF_SMS' );
		}
		$response = json_decode( $response );
		if ( ! empty( $response->data ) && ! empty( $response->data->access_token ) ) {
			$access_token = trim( $response->data->access_token );
		}
		if ( empty( $access_token ) ) {
			return self::errors( $response );
		}


		if ( $action == "send" ) {

			//$to       = str_ireplace( '+', '', $to );
			//$messages = rawurlencode( $messages );

			$data = array(
				'recipients'  => $to,
				'senderName'  => $from,
				'messageText' => $messages,
				'messageType' => 'text',
			);
			$ch   = curl_init( 'http://api.cequens.com/cequens/api/v1/messaging' );
			curl_setopt( $ch, CURLOPT_HTTPHEADER, array(
				'Content-Type:application/json',
				'Authorization: Bearer ' . $access_token
			) );
			curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode( $data ) );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
			$response = curl_exec( $ch );
			curl_close( $ch );
			if ( empty( $response ) ) {
				return __( 'Unknown Error.', 'GF_SMS' );
			}
			$response = json_decode( $response );

			if ( isset( $response->replyCode ) && $response->replyCode == '0' ) {
				return 'OK';
			}
			if ( isset( $response->replyMessage ) && stripos( $response->replyMessage, 'successfully' ) !== false ) {
				return 'OK';
			}
			return self::errors( $response, 'SendSms' );
		}


		if ( $action == "credit" ) {

			$ch = curl_init( 'http://api.cequens.com/cequens/api/v1/accounts/balance' );
			curl_setopt( $ch, CURLOPT_HTTPHEADER, array(
				'Content-Type:application/json',
				'Authorization: Bearer ' . $access_token
			) );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );

			$response = curl_exec( $ch );
			curl_close( $ch );
			if ( empty( $response ) ) {
				return __( 'Unknown Error.', 'GF_SMS' );
			}
			$response = json_decode( $response );

			$balances = array();
			if ( ! empty( $response->data ) && ! empty( $response->data->Account ) ) {
				$balances[ $response->data->Account->UserName ] = $response->data->Account->Credit;
			}
			if ( ! empty( $response->data ) && ! empty( $response->data->SubAccounts ) ) {
				foreach ( $response->data->SubAccounts as $sub_account ) {
					$balances[ $sub_account->UserName ] = $sub_account->Credit;
				}
			}
			if ( ! empty( $balances ) ) {
				if ( isset( $balances[ $userName ] ) ) {
					return $balances[ $userName ];
				}
			}

			return self::errors( $response, 'GetBalance' );
		}

		if ( $action == "range" ) {
			$min = 1000;
			$max = 5000;

			return array( "min" => $min, "max" => $max );
		}

	}

	private static function errors( $response, $method = 'SignIn' ) {
		$error = array();

		if ( ! empty( $response->replyMessage ) ) {
			$error[] = trim( str_ireplace( '[Request failed]', '', $response->replyMessage ) );
		}
		if ( ! empty( $response->error ) && ! empty( $response->error->description ) ) {
			$error[] = $response->error->description;
		}
		if ( ! empty( $response->error->internalErrors[0] ) && ! empty( $response->error->internalErrors[0]->details ) ) {
			$error[] = $response->error->internalErrors[0]->details;
		}

		if ( ! empty( $error ) ) {
			return $method . ' ::: ' . implode( ' ::: ', $error );
		} else {
			return $method . ' ::: ' . __( 'Unknown Error.', 'GF_SMS' );
		}
	}

}