<?php

/**
 * Class Name : GFHANNANSMS_Pro_{strtoupper( php file name) }
 */
class GFHANNANSMS_Pro_CLICKATELL_PLATFORM {

	/*
	* Gateway title	
	*/
	public static function name( $gateways ) {

		$name = __( 'Clickatell (Platform)', 'GF_SMS' );

		$gateway = array( strtolower( str_replace( 'GFHANNANSMS_Pro_', '', get_called_class() ) ) => $name );

		return array_unique( array_merge( $gateways, $gateway ) );
	}

	/*
	* Gateway parameters
	*/
	public static function options() {
		return array(
			'api_key' => __( 'API Key', 'GF_SMS' ),
		);
	}

	/*
	* Gateway credit
	*/
	public static function credit() {
		return false;
	}

	/*
	* Gateway action
	*/
	public static function process( $options, $action, $from, $to, $message ) {

		if ( $action == 'credit' && ! self::credit() ) {
			return false;
		}

		$to = explode( ',', $to );
		$to = array_map( 'trim', $to );

		$api_key = $options['api_key'];
		$message = urlencode( $message );

		if ( $action == "send" ) {

			$parameters = array(
				'content' => $message,
				'to'      => $to,
			);

			if ( ! empty( $from ) ) {
				$parameters['from'] = $from;
			}

			$response = wp_remote_post( 'https://platform.clickatell.com/messages', array(
				'headers' => array(
					'Content-Type'  => 'application/json',
					'Authorization' => $api_key,
				),
				'body'    => json_encode( $parameters )
			) );

			if ( is_wp_error( $response ) ) {
				return $response->get_error_message();
			} else {

				if ( empty( $response['body'] ) ) {
					return __( 'Unknow Error.', 'GF_SMS' );
				}

				$response = (array) json_decode( $response['body'] );

				if ( ! empty( $response['error'] ) ) {
					return $response['error'];
				}

				if ( empty( $response['messages'] ) ) {
					return __( 'Unknow Error.', 'GF_SMS' );
				}

				$errors = array();

				foreach ( $response['messages'] as $result ) {

					$result = (array) $result;

					$to = ! empty( $result['to'] ) ? $result['to'] . ':::=>' : '';

					if ( ! empty( $result['error'] ) ) {
						$errors[] = $to . $result['error'];
					} else if ( empty( $result['accepted'] ) || ! $result['accepted'] ) {
						$errors[] = $to . __( 'Unknow Error.', 'GF_SMS' );
					}

				}

				if ( ! empty( $errors ) ) {
					return implode( ' | ', $errors );
				}

				return 'OK';
			}

		}


		if ( $action == "credit" ) {
		}

		if ( $action == "range" ) {
			$min = 5;
			$max = 20;

			return array( "min" => $min, "max" => $max );
		}

	}
}