<?php
/**
 * Class Message
 * @package Unifonic\API\Message
 */
if ( !class_exists('UnifonicMessage') ) :
    Class UnifonicMessage
    {

        /**
         * @var
         */
        private $client;


        /**
         * @param $oUnifonicClient
         */
        public function __construct( $oUnifonicClient )
        {
            $this->client = $oUnifonicClient;

        }

        public function Rules( $methodName )
        {

            $rules = array(
                'Send' => array(
                    'Recipient' => 'numeric|required|min_len,12|max_len,14',
                    'Body' => 'required',
                    'SenderID' => 'max_len,16',
                ),
                'SendBulkMessages' => array(
                    'Recipient' => 'required',
                    'Body' => 'required',
                    'SenderID' => 'max_len,16',
                ),
                'GetMessageIDStatus' => array(
                    'MessageID' => 'numeric|required',
                ),
                'GetMessagesDetails' => array(
                    'MessageID' => 'numeric',
                    'status' => 'min_len,12|max_len,14',
                    'SenderID' => 'max_len,16',
                    'DateFrom' => 'max_len,55',
                    'DateFTo' => 'max_len,55',
                    'limit' => 'numeric',
                    'page' => 'numeric',
                ),
                'GetMessagesReport' => array(
                    'DateFrom' => 'max_len,55',
                    'DateFTo' => 'max_len,55',
                ),
                'GetScheduled' => array(),
                'StopScheduled' => array(
                    'Recipient' => 'numeric|required|'
                )
            );

            return $rules["$methodName"];

        }

        /**
         * @param $Recipient
         * @param $Body
         * @param $SenderID
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function Send( $Recipient, $Body, $SenderID = null )
        {
            try {

                $aParams = array(
                    'Recipient' => $Recipient,
                    'Body' => $Body,
                    'SenderID' => $SenderID
                );

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_Send($aParams);
                } else {
                    return $valid[0];
                }


            } catch (Exception $e) {
                throw $e;
            }
        }

        /**
         * @param $Recipient
         * @param $Body
         * @param $SenderID
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function SendBulkMessages( $Recipient, $Body, $SenderID = null )
        {
            try {
                $aParams = array(
                    'Recipient' => $Recipient,
                    'Body' => $Body,
                    'SenderID' => $SenderID
                );

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_SendBulk($aParams);
                } else {
                    return $valid[0];
                }

            } catch (Exception $e) {
                throw $e;
            }
        }

        /**
         * @param $MessageID
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function GetMessageIDStatus( $MessageID )
        {
            try {

                $aParams = array('MessageID' => $MessageID);

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_GetMessageIDStatus($aParams);
                } else {
                    return $valid[0];
                }


            } catch (Exception $e) {
                throw $e;
            }
        }


        /**
         * @param $MessageID
         * @param $status
         * @param $SenderID
         * @param $DateFrom
         * @param $DateTo
         * @param $limit
         * @param $page
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function GetMessagesDetails( $MessageID = null, $status = null, $SenderID = null, $DateFrom = null, $DateTo = null, $limit = null, $page = null )
        {

            try {
                $aParams = array(
                    'MessageID' => $MessageID,
                    'status' => $status,
                    'SenderID' => $SenderID,
                    'DateFrom' => $DateFrom,
                    'DateTo' => $DateTo,
                    'Limit' => $limit,
                    'page' => $page
                );

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_GetMessagesDetails($aParams);
                } else {
                    return $valid[0];
                }

            } catch (Exception $e) {
                throw $e;
            }
        }

        /**
         * @param $DateFrom
         * @param $DateTo
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function GetMessagesReport( $DateFrom = null, $DateTo = null )
        {
            try {
                $aParams = array('DateFrom' => $DateFrom, 'DateTo' => $DateTo);

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_GetMessagesReport($aParams);
                } else {
                    return $valid[0];
                }

            } catch (Exception $e) {
                throw $e;
            }
        }

        /**
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function GetScheduled()
        {
            try {
                $aParams = array();

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_GetScheduled($aParams);
                } else {
                    return $valid[0];
                }

            } catch (Exception $e) {
                throw $e;
            }
        }

        /**
         * @return mixed
         * @throws \Exception
         * @throws \Unifonic\API\Exception
         */
        public function StopScheduled( $MessageID )
        {
            try {
                $aParams = array('MessageID' => $MessageID);

                $valid = UnifonicClient::is_valid($aParams, $this->Rules(__FUNCTION__));
                if ( $valid === true ) {
                    return $this->client->Messages_StopScheduled($aParams);
                } else {
                    return $valid[0];
                }

            } catch (Exception $e) {
                throw $e;
            }
        }
    }
endif;