<?php

/*
 * Class Name : GFHANNANSMS_Pro_{strtoupper( php file name) }	
 */

class GFHANNANSMS_Pro_MSG91
{

    /*
    * Gateway title
    */
    public static function name( $gateways )
    {

        $name = __('Msg91', 'GF_SMS');

        $gateway = array(strtolower(str_replace('GFHANNANSMS_Pro_', '', get_called_class())) => $name);
        return array_unique(array_merge($gateways, $gateway));
    }


    /*
    * Gateway parameters
    */
    public static function options()
    {
        return array(
            'authKey' => __('Authentication Key', 'GF_SMS'),
            'route' => __('Route', 'GF_SMS'),
        );
    }

    /*
    * Gateway credit
    */
    public static function credit()
    {
        return true;
    }

    /*
    * Gateway action
    */
    public static function process( $options, $action, $from, $to, $messages )
    {

        if ( $action == 'credit' && !self::credit() ) {
            return false;
        }

        if ( !extension_loaded('curl') )
            return __('Please enable cURL extension in PHP', 'GF_SMS');

        $authKey = $options['authKey'];
        $route = $options['route'];

        if ( $action == "send" ) {

            $to = str_ireplace('+', '', $to);

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => 'http://api.msg91.com/api/sendhttp.php',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => array(
                    'authkey' => $authKey,
                    'mobiles' => $to,
                    'message' => urlencode($messages),
                    'sender' => $from,
                    'route' => $route,
                    'response' => 'json',
                ),
            ));
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            $response = curl_exec($ch);

            if ( curl_errno($ch) )
                return curl_error($ch);

            curl_close($ch);

            $response = (array)json_decode($response);

            if ( !empty($response['type']) && strtolower($response['type']) == 'success' )
                return 'OK';

            if ( !empty($response['message']) )
                return $response['message'];

            return __('Unknown Error.', 'GF_SMS');
        }


        if ( $action == "credit" ) {

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => 'http://api.msg91.com/api/balance.php',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => array(
                    'authkey' => $authKey,
                    'type' => $route,
                ),
            ));
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            $response = curl_exec($ch);

            if ( curl_errno($ch) )
                return curl_error($ch);

            curl_close($ch);

            $response = (array)json_decode($response);

            if ( !empty($response['msg']) ) {
                return self::response($response['msg']);
            }

            if ( !empty($response[0]) ) {
                return $response[0];
            }

            return $response;

        }

        if ( $action == "range" ) {
            $min = 100;
            $max = 300;
            return array("min" => $min, "max" => $max);
        }

    }


    public static function response( $status )
    {

        switch ( $status ) {

            case '101' :
                $message = __('Missing mobile no.', 'GF_SMS');
                break;

            case '102' :
                $message = __('Missing message', 'GF_SMS');
                break;

            case '103' :
                $message = __('Missing sender ID', 'GF_SMS');
                break;

            case '104' :
                $message = __('Missing username', 'GF_SMS');
                break;

            case '105' :
                $message = __('Missing password', 'GF_SMS');
                break;

            case '106' :
                $message = __('Missing Authentication Key', 'GF_SMS');
                break;

            case '107' :
                $message = __('Missing Route', 'GF_SMS');
                break;

            case '202' :
                $message = __('Invalid mobile number. You must have entered either less than 10 digits or there is an alphabetic character in the mobile number field in API.', 'GF_SMS');
                break;

            case '203' :
                $message = __('Invalid sender ID. Your sender ID must be 6 characters, alphabetic.', 'GF_SMS');
                break;

            case '207' :
                $message = __('Invalid authentication key. Crosscheck your authentication key from your account’s API section.', 'GF_SMS');
                break;

            case '208' :
                $message = __('IP is blacklisted. We are getting SMS submission requests other than your whitelisted IP list. ', 'GF_SMS');
                break;

            case '205' :
                $message = __('This route is dedicated for high traffic. You should try with minimum 20 mobile numbers in each request', 'GF_SMS');
                break;

            case '209' :
                $message = __('Default Route for dialplan not found', 'GF_SMS');
                break;

            case '210' :
                $message = __('Route could not be determined', 'GF_SMS');
                break;

            case '301' :
                $message = __('Insufficient balance to send SMS', 'GF_SMS');
                break;

            case '302' :
                $message = __('Expired user account. You need to contact your account manager.', 'GF_SMS');
                break;

            case '303' :
                $message = __('Banned user account', 'GF_SMS');
                break;

            case '306' :
                $message = __('This route is currently unavailable. You can send SMS from this route only between 9 AM - 9 PM.', 'GF_SMS');
                break;

            case '307' :
                $message = __('Incorrect scheduled time', 'GF_SMS');
                break;

            case '308' :
                $message = __('Campaign name cannot be greater than 32 characters', 'GF_SMS');
                break;

            case '309' :
                $message = __('Selected group(s) does not belong to you', 'GF_SMS');
                break;

            case '310' :
                $message = __('SMS is too long. System paused this request automatically.', 'GF_SMS');
                break;

            case '311' :
                $message = __('Request discarded because same request was generated twice within 10 seconds', 'GF_SMS');
                break;

            case '418' :
                $message = __('IP is not whitelisted', 'GF_SMS');
                break;

            case '505' :
                $message = __('Your account is a demo account. Please contact support for details', 'GF_SMS');
                break;

            case '506' :
                $message = __('Small campaign limit exceeded. (only 20 campaigns of less than 100 SMS in 24 hours can be sent, exceeding it will show the error)', 'GF_SMS');
                break;

            case '001' :
                $message = __('Unable to connect database', 'GF_SMS');
                break;

            case '002' :
                $message = __('Unable to select database', 'GF_SMS');
                break;

            case '601' :
                $message = __('Internal error.Please contact support for details', 'GF_SMS');
                break;

            default :
                $message = $status;
                break;
        }

        return $message;
    }
}