<?php

/*
 * Class Name : GFHANNANSMS_Pro_{strtoupper( php file name) }	
 */

class GFHANNANSMS_Pro_SMSS360 {

	/*
	* Gateway title	
	*/
	public static function name( $gateways ) {

		$name = __( 'SMSS360', 'GF_SMS' );

		$gateway = array( strtolower( str_replace( 'GFHANNANSMS_Pro_', '', get_called_class() ) ) => $name );

		return array_unique( array_merge( $gateways, $gateway ) );
	}


	/*
	* Gateway parameters
	*/
	public static function options() {
		return array(
			'email' => __( 'SMSS360 Email', 'GF_SMS' ),
			'key'   => __( 'SMSS360 Key', 'GF_SMS' ),
		);
	}

	/*
	* Gateway credit
	*/
	public static function credit() {
		return false;
	}

	/*
	* Gateway action
	*/
	public static function process( $options, $action, $from, $to, $messages ) {

		if ( $action == 'credit' && ! self::credit() ) {
			return false;
		}

		$key   = $options['key'];
		$email = $options['email'];

		if ( $action == "send" ) {

			$content = 'https://www.smss360.com/api/sendsms.php?email=' . $email . '&key=' . $key . '&recipient=' . $to . '&message=' . rawurlencode( $messages );

			if ( extension_loaded( 'curl' ) ) {
				$ch = curl_init( $content );
				curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
				$response = curl_exec( $ch );
				curl_close( $ch );
			} else {
				$response = file_get_contents( $content );
			}

			if ( empty( $response ) || ( stripos( $response, 'statusMsg' ) === false && stripos( $response, 'statusCode' ) === false ) ) {
				return __( 'unknown error.', 'GF_SMS' );
			}

			$statusMsg = '';
			preg_match( '/\<statusMsg\>(.*)\<\/statusMsg\>/is', $response, $matches );
			if ( ! empty( $matches[1] ) ) {
				$statusMsg = $matches[1];
			}

			$statusCode = '';
			preg_match( '/\<statusCode\>(.*)\<\/statusCode\>/is', $response, $matches );
			if ( ! empty( $matches[1] ) ) {
				$statusCode = $matches[1];
			}

			if ( in_array( $statusCode, array( '0', '1606' ) ) || stripos( $statusMsg, 'accepted' ) !== false || stripos( $statusMsg, 'successfull' ) !== false ) {
				return 'OK';
			}

			if ( ! empty( $statusMsg ) ) {
				return $statusMsg;
			}

			return self::response( $statusCode );

		}


		if ( $action == "credit" ) {

		}

		if ( $action == "range" ) {
			$min = 5000;
			$max = 10000;

			return array( "min" => $min, "max" => $max );
		}

	}


	public static function response( $status ) {

		switch ( $status ) {

			case '1126' :
				$message = __( 'Your account is under the verifying progress. Please contact us for more information.', "GF_SMS" );
				break;

			case '1204' :
				$message = __( 'The query string is empty.', "GF_SMS" );
				break;

			case '1206' :
				$message = __( 'Please enter your email address.', "GF_SMS" );
				break;

			case '1207' :
				$message = __( 'Please enter your key.', "GF_SMS" );
				break;

			case '1208' :
				$message = __( 'Invalid email or key.', "GF_SMS" );
				break;

			case '1209' :
				$message = __( 'Please activate your account.', "GF_SMS" );
				break;

			case '1210' :
				$message = __( 'Account has been suspended.', "GF_SMS" );
				break;

			case '1602' :
				$message = __( 'Invalid recipient(s).', "GF_SMS" );
				break;

			case '1603' :
				$message = __( 'Please enter your message.', "GF_SMS" );
				break;

			case '1605' :
				$message = __( 'Maximum 10 SMS per recipient(s).', "GF_SMS" );
				break;

			case '1606' :
				$message = __( 'SMS accepted.', "GF_SMS" );
				break;

			case '1608' :
				$message = __( 'Insufficient balance.', "GF_SMS" );
				break;

			case '1609' :
				$message = __( 'Please enter at least one recipient.', "GF_SMS" );
				break;

			default:
				$message = __( 'unknown error.', 'GF_SMS' );

		}

		return $message;
	}
}