<?php

/**
 * Class Name : GFHANNANSMS_Pro_{strtoupper( php file name) }
 */
class GFHANNANSMS_Pro_UNIFONIC
{

    /*
    * Gateway title
    */
    public static function name( $gateways )
    {

        $name = __('Unifonic', 'GF_SMS');

        $gateway = array(strtolower(str_replace('GFHANNANSMS_Pro_', '', get_called_class())) => $name);
        return array_unique(array_merge($gateways, $gateway));
    }

    /*
    * Gateway parameters
    */
    public static function options()
    {
        return array(
            'AppSid' => __('AppSid', 'GF_SMS'),
        );
    }

    /*
    * Gateway credit
    */
    public static function credit()
    {
        return true;
    }

    /*
    * Gateway action
    */
    public static function process( $options, $action, $from, $to, $message )
    {

        if ( $action == 'credit' && !self::credit() ) {
            return false;
        }

        $AppSid = $options['AppSid'];

        if ( empty($AppSid) )
            return __('AppSid is required.', 'GF_SMS');

        if ( !class_exists('UnifonicClient') )
            include_once( 'lib/Unifonic/Client.php' );


        if ( $action == "send" ) {

            try {

                if ( empty($message) )
                    return __('Message Body is required.', 'GF_SMS');

                if ( empty($to) )
                    return __('Recipient Body is required.', 'GF_SMS');

                $to = str_ireplace('+', '', $to);

                $client = new UnifonicClient($AppSid);
                $response = $client->Messages->SendBulkMessages($to, $message, $from);
                $Messages = !empty($response->Messages) ? $response->Messages : array();
                $Messages = array_map(array(__CLASS__, 'object_to_array'), $Messages);
                $statuses = array_column($Messages, 'Status');
                $statuses = array_map('strtolower', $statuses);

                if ( !in_array('failed', $statuses) && !in_array('rejected', $statuses) )
                    return 'OK';

                return __('Unknow error.', 'GF_SMS');

            } catch (Exception $e) {
                return $e->getMessage();
            }

        }


        if ( $action == "credit" ) {

            try {

                $client = new UnifonicClient($AppSid);
                $response = $client->Account->GetBalance();
                $Balance = !empty($response->Balance) ? $response->Balance : '';
                $CurrencyCode = !empty($response->CurrencyCode) ? $response->CurrencyCode : '';

                return $Balance . ' ' . $CurrencyCode;

            } catch (Exception $e) {
                return $e->getMessage();
            }
        }

        if ( $action == "range" ) {
            $min = 100;
            $max = 300;
            return array("min" => $min, "max" => $max);
        }

    }

    public static function object_to_array( $object )
    {
        return (array)$object;
    }
}