<?php

if ( ! class_exists( 'GFForms' ) ) {
	die();
}

/**
 * Gravity Form field for Rating Item
 * 
 * @author dpowney
 *
 */
class MRP_GF_Field_Rating_Item extends GF_Field {

	/**
	 * @var string $type The field type.
	 */
	public $type = null;
	
	/**
	 * The rating item
	 * @var unknown
	 */
	public $rating_item = null;
	
	/**
	 * 
	 * @param unknown $data
	 */
	function __construct( $data ) {
		
		parent::__construct( $data );
		
		$rating_item_id = substr( $this->type, 12 ); // "rating_item_X"
		
		$rating_items = MRP_Multi_Rating_API::get_rating_items();
		$rating_item = isset( $rating_items[$rating_item_id] ) ? $rating_items[$rating_item_id] : null;
		
		if ( $rating_item ) {
			$this->rating_item = $rating_item;
			$this->label = $rating_item['description'];
		}
		
	}

	/**
	 * Return the field title, for use in the form editor.
	 *
	 * @return string
	 */
	public function get_form_editor_field_title() {
		return $this->rating_item['description'];
	}

	/**
	 * Assign the field button to the Advanced Fields group.
	 *
	 * @return array
	 */
	public function get_form_editor_button() {
		return array(
			'group' => 'advanced_fields',
			'text'  => $this->get_form_editor_field_title(),
		);
	}

	/**
	 * The settings which should be available on the field in the form editor.
	 *
	 * @return array
	 */
	function get_form_editor_field_settings() {
		return array(
			'rules_setting',
			'css_class_setting',
			'visibility_setting' // for adminOnly
		);
	}

	/**
	 * Enable this field for use with conditional logic.
	 *
	 * @return bool
	 */
	public function is_conditional_logic_supported() {
		return false;
	}

	/**
	 * The scripts to be included in the form editor.
	 *
	 * @return string
	 */
	public function get_form_editor_inline_script_on_page_render() {

		// set the default field label for the rating_item type field
		$script = sprintf( "function SetDefaultValues_rating_item(field) {field.label = '%s';}", $this->get_form_editor_field_title() ) . PHP_EOL;

		// initialize the fields custom settings
		$script .= "jQuery(document).bind('gform_load_field_settings', function (event, field, form) {" .
		           "var inputClass = field.inputClass == undefined ? '' : field.inputClass;" .
		           "jQuery('#input_class_setting').val(inputClass);" .
		           "});" . PHP_EOL;

		// saving the rating_item setting
		$script .= "function SetInputClassSetting(value) {SetFieldProperty('inputClass', value);}" . PHP_EOL;

		return $script;
	}

	/**
	 * Define the fields inner markup.
	 *
	 * @param array $form The Form Object currently being processed.
	 * @param string|array $value The field value. From default/dynamic population, $_POST, or a resumed incomplete submission.
	 * @param null|array $entry Null or the Entry Object currently being edited.
	 *
	 * @return string
	 */
	public function get_field_input( $form, $value = '', $entry = null ) {
		
		$id              = absint( $this->id );
		$form_id         = absint( $form['id'] );
		$is_entry_detail = $this->is_entry_detail();
		$is_form_editor  = $this->is_form_editor();

		// Prepare the value of the input ID attribute.
		$field_id = $is_entry_detail || $is_form_editor || $form_id == 0 ? "input_$id" : 'input_' . $form_id . "_$id";

		$value = esc_attr( $value );

		// Get the value of the inputClass property for the current field.
		$inputClass = $this->inputClass;

		// Prepare the input classes.
		$size         = $this->size;
		$class_suffix = $is_entry_detail ? '_admin' : '';
		$class        = $size . $class_suffix . ' ' . $inputClass;

		$option_value_text_lookup = array();
		if ( isset( $this->rating_item['option_value_text'] ) ) {
			$option_value_text_lookup = MRP_Utils::get_option_value_text_lookup( $this->rating_item['option_value_text'] );
		}
		
		// Note "input_{$id}" field value is mapped back before submission via gform_pre_process filter
				
		ob_start();
		mrp_get_template_part( 'rating-form', 'rating-item', true, array(
				'rating_item_id' => $this->rating_item['rating_item_id'],
				'element_id' => 'rating-item-' . $this->rating_item['rating_item_id'] . '-' . $form['id'],  
				'description' => $this->rating_item['description'],
				'max_option_value' => $this->rating_item['max_option_value'],
				'default_option_value' => $this->rating_item['default_option_value'],
				'required' => $this->isRequired, 
				'option_value_text' => $this->rating_item['option_value_text'],
				'class' => "{$class}",
				'style' => null,
				'rating_item_type' => $this->rating_item['type'],
				'option_value_text_lookup' => $option_value_text_lookup,
				'rating_entry_id' => null,
				'only_show_text_options' => $this->rating_item['only_show_text_options'],
				'show_rating_item_label' => false
		) );
			
		$input = ob_get_contents();
		ob_end_clean();
		
		$input = apply_filters( 'mrp_template_html', $input );
		
		return sprintf( "<div class='ginput_container ginput_container_%s'>%s</div>", $this->type, $input );
	}
	
	/**
	 * (non-PHPdoc)
	 * @see GF_Field::validate()
	 */
	public function validate( $value, $form ) {
		// TODO validate rating item
	}
	
	// TODO weights in general settings
}
