<?php
/*
 Plugin Name: Multi Rating Pro - Gravity Forms
 Plugin URI: http://multiratingpro.com
 Description: Gravity Forms add-on for Multi Rating Pro. Take advantage of Gravity Forms integrated with a powerful rating system.
 Version: 1.0.1
 Author: Daniel Powney
 Author URI: http://multiratingpro.com
 License: GPL2
 Text Domain: mrp-gravity-forms
 Domain Path: languages

 Multi Rating Pro - Gravity Forms is distributed under the terms of the GNU General 
 Public License as published by the Free Software Foundation, either version 2 of the 
 License, or any later version.

 Multi Rating Pro - Gravity Forms is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 <http://www.gnu.org/licenses/>
*/

define( 'MRP_GRAVITY_FORMS_ADDON_VERSION', '1.0.1' );

if ( ! class_exists( 'GFAddOn' ) ) {
	return;
}

/**
 * Multi Rating Pro - Gravity Forms plugin main class. This class extends the Gravity Forms 
 * add-on framework.
 * 
 * @author dpowney
 *
 */
class MRP_Gravity_Forms extends GFAddOn {

	protected $_version = MRP_GRAVITY_FORMS_ADDON_VERSION;
	protected $_min_gravityforms_version = '1.9';
	protected $_slug = 'mrp-gravity-forms';
	protected $_path;
	protected $_full_path = __FILE__;
	protected $_title = 'Multi Rating Pro';
	protected $_short_title = 'Multi Rating Pro';

	/**
	 * @var object $_instance If available, contains an instance of this class.
	 */
	private static $_instance;


	/**
	 * Returns an instance of this class, and stores it in the $_instance property.
	 *
	 * @return object $_instance An instance of this class.
	 */
	public static function get_instance() {
		if ( self::$_instance == null ) {
			self::$_instance = new self();
			self::$_instance->_path = 'multi-rating-pro-gravity-forms' . DIRECTORY_SEPARATOR . 'mrp-gravity-forms.php';			
		}

		return self::$_instance;
	}
	
	
	/**
	 * Activates the plugin
	 */
	public static function activate_plugin() {
	
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	
		try {
	
			global $wpdb, $charset_collate;
				
			$sql = 'CREATE TABLE ' . $wpdb->prefix . 'mrp_gravity_forms_form (
					rating_form_id bigint(20) NOT NULL,
					gform_form_id mediumint(8) NOT NULL,
					PRIMARY KEY  (rating_form_id,gform_form_id)
			) ' . $charset_collate;
			dbDelta( $sql );
				
			$wpdb->show_errors();
			
			$sql = 'CREATE TABLE ' . $wpdb->prefix . 'mrp_gravity_forms_entry (
					rating_entry_id bigint(20) NOT NULL,
					gform_lead_id mediumint(20) NOT NULL,
					PRIMARY KEY  (rating_entry_id,gform_lead_id)
			) ' . $charset_collate;
			dbDelta( $sql );
	
		} catch ( Exception $e ) {
			// do nothing
		}
	}

	/**
	 * Include gravity forms field for rating item early so it is available when entry exports are being performed.
	 */
	public function pre_init() {
		
		parent::pre_init();

		if ( $this->is_gravityforms_supported() && class_exists( 'GF_Field' ) ) {
			require_once( 'includes/class-gf-field-rating-item.php' );
		}
	}

	/**
	 * Inits plugin
	 */
	public function init() {
		
		parent::init();
		
		// register rating items
		if ( class_exists( 'MRP_Multi_Rating_API' ) ) {
			
			$rating_items = MRP_Multi_Rating_API::get_rating_items();
			
			foreach ( $rating_items as $rating_item ) {

				$rating_item_gf_field = new MRP_GF_Field_Rating_Item( array(
						'type' => 'rating_item_' . $rating_item['rating_item_id']
				) );
				
				GF_Fields::register( $rating_item_gf_field );
			}
		}

		add_action( 'admin_init',  array( MRP_Gravity_Forms::get_instance(), 'plugin_updater' ) );
		add_filter( 'mrp_license_items',  array( MRP_Gravity_Forms::get_instance(), 'license_items' ) );
		add_filter( 'mrp_license_settings_sections',  array( MRP_Gravity_Forms::get_instance(), 'license_settings_sections' ) );
		add_filter( 'mrp_license_setting_fields',  array( MRP_Gravity_Forms::get_instance(), 'license_setting_fields' ) );
		
		add_action( 'gform_pre_process', array( MRP_Gravity_Forms::get_instance(), 'prepare_rating_item_inputs' ), 10, 1 );
		add_action( 'gform_after_submission', array( MRP_Gravity_Forms::get_instance(), 'save_rating_entry' ), 10, 2 );
		add_filter( 'gform_validation', array( MRP_Gravity_Forms::get_instance(), 'validate_rating_entry' ) );
		add_filter( 'gform_validation_message', array( MRP_Gravity_Forms::get_instance(), 'apply_rating_validation_messages' ), 10, 2);
		add_action( 'gform_after_save_form', array( MRP_Gravity_Forms::get_instance(), 'after_save_form' ), 10, 2 ); // why can't I add this one in admin init?
		add_action( 'gform_post_update_form_meta', array( MRP_Gravity_Forms::get_instance(), 'update_form_settings' ), 10, 2 );
		add_filter( 'mrp_rating_form_callback', array( MRP_Gravity_Forms::get_instance(), 'rating_form_callback'), 10, 3 );
		add_action( 'mrp_rating_entry_details_after_custom_fields', array( MRP_Gravity_Forms::get_instance(), 'gform_fields_display' ) ,10, 1);
		add_filter( 'gform_pre_render', array( MRP_Gravity_Forms::get_instance(), 'setup_rating_form_sequence' ), 10, 1 );
		
		add_action( 'plugins_loaded', array( MRP_Gravity_Forms::get_instance(), 'load_textdomain' ) );
		
		add_action( 'gform_delete_lead', array( MRP_Gravity_Forms::get_instance(), 'delete_gform_entry' ), 10, 1 );

		add_action( 'mrp_after_delete_rating_entry_success', array( MRP_Gravity_Forms::get_instance(), 'delete_rating_entry' ), 10, 1 );
		
		add_action( 'gform_after_delete_form', array( MRP_Gravity_Forms::get_instance(), 'delete_gform' ) );
		add_action( 'mrp_after_delete_rating_form', array( MRP_Gravity_Forms::get_instance(), 'delete_rating_form' ) );
		
		if ( $this->is_preview() ) {
			add_action( 'gform_preview_footer', array( MRP_Gravity_Forms::get_instance(), 'preview_footer' ), 10, 1 );
			add_action( 'gform_enqueue_scripts', array( MRP_Gravity_Forms::get_instance(), 'preview_assets' ) );
		}

	}
	
	/**
	 * Admin init plugin
	 */
	public function init_admin() {
		
		parent::init_admin();

		add_filter( 'gform_entry_field_value', array( MRP_Gravity_Forms::get_instance(), 'format_entry_rating_item_value' ), 10, 4 );
		add_filter( 'gform_entries_field_value', array( MRP_Gravity_Forms::get_instance(), 'format_entries_rating_item_value' ), 10, 4 );
		add_filter( 'gform_export_field_value', array( MRP_Gravity_Forms::get_instance(), 'format_entries_rating_item_value' ), 10, 4 );
		add_filter( 'gform_merge_tag_filter', array( MRP_Gravity_Forms::get_instance(), 'format_email_rating_item_value' ), 10, 4 );
		
		// settings
		add_filter( 'mrp_settings_tabs', array( MRP_Gravity_Forms::get_instance(), 'add_gravity_forms_settings_tab' ), 10, 1 );
		add_action( 'mrp_settings_form', array( MRP_Gravity_Forms::get_instance(), 'add_mrp_gravity_forms_settings_form' ), 10, 1 );
		add_action( 'admin_init', array( MRP_Gravity_Forms::get_instance(), 'register_mrp_gravity_forms_settings' ) );
		add_action( 'admin_init', array( MRP_Gravity_Forms::get_instance(), 'plugin_updater' ) );
		
		add_filter( 'mrp_edit_rating_form_page', array( MRP_Gravity_Forms::get_instance(), 'gform_edit_rating_form_page' ), 10, 2 );
		//add_filter( 'mrp_view_entries_page', array( MRP_Gravity_Forms::get_instance(), 'gform_view_entries_page' ), 10, 2 );
		
		add_filter( 'mrp_rating_forms_shortcode_column', array( MRP_Gravity_Forms::get_instance(), 'rating_forms_shortcode_column' ), 10, 2 );
		
		add_filter( 'gform_admin_pre_render', array( MRP_Gravity_Forms::get_instance(), 'gform_admin_pre_render'), 10, 1 );
	
		MRP_Gravity_Forms::get_instance()->load_settings();
	}
	
	/**
	 * Loads plugin text domain
	 */
	public function load_textdomain() {
		load_plugin_textdomain( 'mrp-gravity-forms', false,
				dirname( plugin_basename( __FILE__ ) ) . DIRECTORY_SEPARATOR . 'languages' );
	}
	
	/**
	 * Ensures rating items can only be added to the form once
	 * @param unknown $form
	 * @return unknown
	 */
	public function gform_admin_pre_render( $form ) {
		
		$rating_items = MRP_Multi_Rating_API::get_rating_items();
			
		$field_ids = array();
		foreach ( $rating_items as $rating_item ) {
			array_push( $field_ids, 'rating_item_' . $rating_item['rating_item_id'] );
		}
		
		echo GFCommon::is_form_editor() ? "
        <script type='text/javascript'>
        gform.addFilter('gform_form_editor_can_field_be_added', function (canFieldBeAdded, type) {

			var fieldIds = " . json_encode( $field_ids ) . ";
					
			if (jQuery.inArray(type, fieldIds) >= 0 ) { // inArray returns an index, not true or false
                if (GetFieldsByType([type]).length > 0) {
                    alert('" . __( 'You are only allowed to add a rating item to the form once.', 'mrp-gravity-forms' ) . "');
                    return false;
                }
            }
            return canFieldBeAdded;
        });
        </script>" : '';
	
		//return the form object from the php hook
		return $form;
	}
	
	/**
	 * Initialises scripts
	 *
	 * @return multitype:
	 */
	public function scripts() {
		
		$scripts = parent::scripts();
		
		if ( class_exists( 'MRP_Multi_Rating' ) ) {
			
			array_push( $scripts, array(
					'handle'  => 'mrp-gravity-forms-script',
					'src'     => plugins_url( 'assets' . DIRECTORY_SEPARATOR . 'js' . DIRECTORY_SEPARATOR . 'frontend.js', __FILE__),
					'version' => MRP_Multi_Rating::VERSION,
					'deps'     => array( 'jquery' ),
					'enqueue'	=> array( array() )
			) );
		}
		
		return $scripts;
		
	}
	
	/**
	 * Adds assets in gform preview since Multi Rating Pro is not added
	 */
	public function preview_assets() {
		
		MRP_Multi_Rating::instance()->assets();
		
	}
	
	/**
	 * Adds header styles in gform preview footer since Multi Rating Pro is not added
	 *
	 * @return multitype:
	 */
	public function preview_footer( $form ) {
		
		MRP_Multi_Rating::instance()->mrp_head();
		
	}

	
	/**
	 * Setup a rating form sequence number. The form id is used as the sequence number. Gravity Forms does not 
	 * support the same form displayed multiple times on the one page.
	 * 
	 * @param unknown $form
	 * @return string
	 */
	public function setup_rating_form_sequence( $form ) {
		
		if ( $form && $this->has_rating_item( $form ) ) {
			array_push( $form, array( 'sequence' => $form['id'] ) );
		}
		
		return $form;
	}
	
	/**
	 * Formats a single rating item value
	 *
	 * @param unknown $value
	 * @param unknown $field
	 * @param unknown $entry
	 * @param unknown $form
	 * @return unknown|string
	 */
	public function format_entry_rating_item_value( $value, $field, $entry, $form ) {
	
		if ( strpos( $field->type, 'rating_item_' ) === false ) {
			return $value;
		}
	
		$rating_item = $field->rating_item;
	
		$option_value_text_lookup = array();
		if ( isset( $rating_item['option_value_text'] ) && class_exists( 'MRP_Utils' ) ) {
			$option_value_text_lookup = MRP_Utils::get_option_value_text_lookup( $rating_item['option_value_text'] );
		}
	
		if ( isset( $option_value_text_lookup[$value] ) ) {
			return $option_value_text_lookup[$value];
		}
	
		return $value;
	}
	
	/**
	 * Formats rating item values as shown in entries page
	 *
	 * @param unknown $value
	 * @param unknown $form_id
	 * @param unknown $field_id
	 */
	public function format_entries_rating_item_value( $value, $form_id, $field_id, $entry ) {
	
		$form = GFAPI::get_form( $form_id );
		$field = RGFormsModel::get_field( $form, $field_id );
	
		return $this->format_entry_rating_item_value( $value, $field, $entry, $form );
	
	}
	
	
	/**
	 * Format rating item fields for e-mails
	 *
	 * @param unknown $value
	 * @param unknown $merge_tag
	 * @param unknown $modifier
	 * @param unknown $field
	 * @return boolean|string|unknown
	 */
	public function format_email_rating_item_value( $value, $merge_tag, $modifier, $field ) {
		return $this->format_entry_rating_item_value( $value, $field, null, null );
	}
	
	/**
	 * Prepares rarting item values in $_POST object
	 * 
	 * @param unknown $form
	 */
	public function prepare_rating_item_inputs( $form ) {
	
		foreach ( $form['fields'] as $field ) {
			if ( strpos( $field->type, 'rating_item_' ) !== false ) {
				$element_id = 'rating-item-' . $field->rating_item['rating_item_id'] . '-' . $form['id'];
				$_POST['input_' . $field->id] = $_POST[$element_id];
				unset( $_POST[$element_id] );
			}
		}
		
		return $form;
	}
		
	/**
	 * Saves MRP entry to the database
	 * 
	 * Based on https://www.gravityhelp.com/documentation/article/gform_after_submission/
	 *
	 * @param unknown $entry
	 * @param unknown $form
	*/
	public function save_rating_entry( $entry, $form ) {
		
		if ( $this->is_preview() || ! class_exists( 'MRP_Multi_Rating' ) || ! $this->has_rating_item( $form ) ) {
			return;
		}
		
		$post_id = url_to_postid( MRP_Utils::get_current_url() );
		
		$post_id = apply_filters( 'mrp_object_id', $post_id, apply_filters( 'mrp_default_language', null ) );
		$rating_form_id = $this->get_rating_form_id( $form['id'] );
		
		if ( $post_id == 0 || $post_id == null ) {
			return; // Nothing to do.
		}
		
		$title = '';
		$name = '';
		$email = '';
		$comment = '';
		
		$review_fields_map = isset( $form['mrp-gravity-forms'] ) ? $form['mrp-gravity-forms'] : array();
		foreach ( $review_fields_map as $field_name => $field_id ) {
			
			/// find field... there must be a more efficient way to do this...
			$field = null;
			foreach ( $form['fields'] as $temp_field ) {
				if ( $temp_field['id'] == $field_id ) {
					$field = $temp_field;
					break;
				}
			}
			
			$field_value = GFCommon::get_lead_field_display( $field, RGFormsModel::get_lead_field_value( $entry, $field ), $entry['currency'] );
			$field_value = apply_filters( 'gform_entry_field_value', $field_value, $field, $entry, $form );
			
			if ( $field_value && strlen( $field_value ) > 0 ) {
				if ( strpos( $field_name, 'title' ) > 0 ) {
					$title = $field_value;
				} else if ( strpos( $field_name, 'name' ) > 0 ) {
					$name = $field_value;
				} else if ( strpos( $field_name, 'email' ) > 0 ) {
					$email = $field_value;
				} else if ( strpos( $field_name, 'comment' ) > 0 ) {
					$comment = $field_value;
				}
			}
		}
	
		$rating_item_values = array();
		foreach ( $form['fields'] as $field ) {
			
			if ( strpos( $field->type, 'rating_item_' ) !== false ) {
				$rating_item_values[$field->rating_item['rating_item_id']] = rgar( $entry, $field->id );
			} else if ( $field->type == 'fileupload' ) {
				
				$url =  $entry[$field->id];
				$parsed_url = parse_url( $url );
				$path = $_SERVER['DOCUMENT_ROOT'] . $parsed_url['path'];
				$image = wp_get_image_editor( $path );
				
				if ( ! is_wp_error( $image ) ) {
					$image_size_meta = $this->get_image_size_meta();
					
					$result = $image->resize( $image_size_meta['width'], $image_size_meta['height'], false ); // false indicates not to crop image
					$result = $image->save( $path );	
				}
			}
		}
		
		global $wp_roles;
		$user = wp_get_current_user();
		$user_id = $user->ID;
		
		// is new so rating entry id is not set
		$rating_entry = array(
				'post_id' => $post_id,
				'rating_form_id' => $rating_form_id,
				'ip_address' => MRP_Utils::get_ip_address(),
				'entry_date' => current_time( 'mysql' ),
				'user_id' => $user_id,
				'rating_item_values' => $rating_item_values,
				'custom_field_values' => array(),
				'title' => $title,
				'name' => $name,
				'email' => $email,
				'comment' => $comment
		);
	
		$save_rating_response = MRP_Multi_Rating_API::save_rating_entry( $rating_entry );
	
		$rating_entry_id = isset( $save_rating_response['rating_entry'] ) ? $save_rating_response['rating_entry']['rating_entry_id'] : null;
	
		if ( $rating_entry_id ) {
			global $wpdb;
			
			$wpdb->insert( $wpdb->prefix . 'mrp_gravity_forms_entry', array(
					'rating_entry_id' => $rating_entry_id,
					'gform_lead_id' => $entry['id'],
			), array( '%d', '%d' ) );
		}
		
	}
	
	/**
	 * Returns image size meta width & height
	 */
	public function get_image_size_meta() {
		
		$image_meta['width'] = 640;
		$image_height['height'] = 400;
		
		$image_sizes = MRP_Utils::get_image_sizes();
					
		$settings = (array) get_option( 'mrp_gravity_forms' );
		
		if ( isset( $image_sizes[$settings['image_size']] ) ) {
			$image_meta = $image_sizes[$settings['image_size']];
			
		} else if ( isset( $image_sizes['thumbnail'] ) ) {
			$image_meta = $image_sizes['thumbnail'];
		}
		
		return $image_meta;
	}
	
	/**
	 * Performs MRP validation if necessary
	 *
	 * @param unknown $validation_result
	 * @return unknown
	*/
	public function validate_rating_entry( $validation_result ) {
		
		if ( $this->is_preview() || ! class_exists( 'MRP_Multi_Rating' ) ) {
			return $validation_result;
		}
		
		$form = $validation_result['form'];

		$rating_item_values = array();
		foreach( $form['fields'] as &$field ) {
			if ( strpos( $field->type, 'rating_item_' ) !== false ) {
				$rating_item_values[$field->rating_item['rating_item_id']] = rgpost( 'input_' . $field->id );
			}
		}
		
		// if MRP validation is not needed (i.e. no rating items), return validation result
		if ( count( $rating_item_values ) == 0 ) {
			return $validation_result;
		}
	
		$rating_form_id = $this->get_rating_form_id( $form['id'] );

		global $wp_roles;
		$user = wp_get_current_user();
		$user_id = $user->ID;
		
		$post_id = url_to_postid( MRP_Utils::get_current_url() );
		
		// is new so rating entry id is not set
		$rating_entry = array(
				'post_id' => $post_id,
				'rating_form_id' => $rating_form_id,
				'ip_address' => MRP_Utils::get_ip_address(),
				'entry_date' => current_time( 'mysql' ),
				'user_id' => $user_id,
				'rating_item_values' => $rating_item_values,
				'custom_field_values' => array()	
		);
	
		// validate rating entry
		$validation_results = MRP_Utils::validate_rating_entry( array(), $rating_entry );
	
		if ( MRP_Utils::has_validation_error( $validation_results ) ) {
			$validation_result['is_valid'] = false;
			$form['mrp_validation_results'] = $validation_results; // so we can access it later :)
		}
	
		$validation_result['form'] = $form;
	
		return $validation_result;
	}
	
	
	/**
	 * Returns MRP error messages if necessary
	 *
	 * @param unknown $message
	 * @param unknown $form
	 * @return string
	*/
	public function apply_rating_validation_messages( $message, $form ) {
		
		if ( ! class_exists( 'MRP_Multi_Rating' ) ) {
			return $message;
		}
	
		if ( isset( $form['mrp_validation_results'] ) ) {
			$message = '<div class="validation_error">';
			foreach ( $form['mrp_validation_results'] as $validation_result ) {
				$message .= $validation_result['message'];
			}
			$message .= '</div>';
		}
	
		return $message;
	}
	
	/**
	 * Deletes link between a Gravity Form and a rating form whenever a Gravity Form is deleted, and will
	 * also delete any associated data if required
	 * 
	 * @param unknown $gform_form_id
	 */
	public function delete_gform( $gform_form_id ) {
		
		// delete associated data
		$settings = (array) get_option( 'mrp_gravity_forms' );
		if ( in_array( 'rating_form', $settings['delete_associated_data'] ) ) {
			$rating_form_id = $this->get_rating_form_id( $gform_form_id );
			// this action hook wont be necessary
			remove_action( 'mrp_after_delete_rating_form', array( MRP_Gravity_Forms::get_instance(), 'delete_rating_form' ) );
			MRP_Multi_Rating_API::delete_rating_form( $rating_form_id );
			MRP_Multi_Rating_API::delete_orphaned_data();
		} else if ( in_array( 'gform_entry', $settings['delete_associated_data'] ) ) {
			$rating_form_id = $this->get_rating_form_id( $gform_form_id );
			$rating_entries = MRP_Multi_Rating_API::get_rating_entries( array( 'rating_form_id' => $rating_form_id ) );
			// this action hook wont be necessary
			remove_action( 'mrp_after_delete_rating_entry_success', array( MRP_Gravity_Forms::get_instance(), 'delete_rating_entry' ) );
			// batch?
			foreach ( $rating_entries as $rating_entry ) {
				MRP_Multi_Rating_API::delete_rating_entry( $rating_entry['rating_entry_id'] );
			}
			MRP_Multi_Rating_API::delete_orphaned_data();
		}
		
		// deletes link
		global $wpdb;
		$wpdb->delete( $wpdb->prefix . 'mrp_gravity_forms_form', array( 'gform_form_id' => $gform_form_id ), array( '%d' ) );
		
	}
	
	/**
	 * Deletes link between a Gravity Form entry and a rating entry whenever a Gravity Form entry is deleted,
	 * and will also delete any associated data if required
	 * 
	 * @param unknown $entry_id
	 */
	public function delete_gform_entry( $entry_id ) {
		
		// delete associated data
		$settings = (array) get_option( 'mrp_gravity_forms' );
		if ( in_array( 'gform_entry', $settings['delete_associated_data'] ) ) {
			$rating_entry_id = $this->get_rating_entry_id( $entry_id );
			// this action hook wont be necessary
			remove_action( 'mrp_after_delete_rating_entry_success', array( MRP_Gravity_Forms::get_instance(), 'delete_rating_entry' ) );
			MRP_Multi_Rating_API::delete_rating_entry( $rating_entry_id );
		}
		
		// delete link last
		global $wpdb;
		$wpdb->delete( $wpdb->prefix . 'mrp_gravity_forms_entry', array( 'gform_lead_id' => $entry_id ), array( '%d' ) );
		
	}
	
	/**
	 * Deletes link between a rating form and a Gravity Form whenever a rating form is deleted, and will
	 * also delete any associated data if required
	 * 
	 * @param unknown $rating_form_id
	 */
	public function delete_rating_form( $rating_form_id ) {
		
		// delete link
		global $wpdb;
		$wpdb->delete( $wpdb->prefix . 'mrp_gravity_forms_form', array( 'rating_form_id' => $rating_form_id ), array( '%d' ) );
		
	}
	
	
	/**
	 *
	 * @param unknown $rating_entry_id
	 */
	public function delete_rating_entry( $params = array() ) {
		
		$rating_entry_id = isset( $params['rating_entry_id'] ) ? $params['rating_entry_id'] : null;
		
		if ( $rating_entry_id == null ) {
			return;
		}
		
		// delete associated data
		$settings = (array) get_option( 'mrp_gravity_forms' );
		if ( in_array( 'rating_entry', $settings['delete_associated_data'] ) ) {
			$entry_id = $this->get_gform_entry_id( $rating_entry_id );
			// this action hook wont be necessary
			remove_action( 'gform_delete_lead', array( MRP_Gravity_Forms::get_instance(), 'delete_gform_entry' ) );
			GFAPI::delete_entry( $entry_id );
		}
		
		// delete link last
		global $wpdb;
		$wpdb->delete( $wpdb->prefix . 'mrp_gravity_forms_entry', array( 'rating_entry_id' => $rating_entry_id ), array( '%d' ) );
		
	
	}
	
	/**
	 * Changes the rating forms shortcode for Gravity Form forms
	 * 
	 * @param unknown $shortcode
	 * @param unknown $rating_form_id
	 * @return string|unknown
	 */
	public function rating_forms_shortcode_column( $shortcode, $rating_form_id ) {
		
		$gform_form_id = $this->get_gform_form_id( $rating_form_id );
		
		if ( $gform_form_id ) {
			return '<code>[gravityform id=' . $gform_form_id . ']</code>';
		}
		
		return $shortcode;
		
	}
	
	/**
	 * Changes the edit rating form page link
	 * 
	 * @param unknown $rating_form_id
	 * @return string
	 */
	public function gform_edit_rating_form_page( $page, $rating_form_id ) {
		
		$gform_form_id = $this->get_gform_form_id( $rating_form_id );
		
		if ( $gform_form_id ) {
			return 'gf_edit_forms&id=' . $gform_form_id; 
		}
		
		return $page;
		
	}
	
	/**
	 * Changes the view entries page link
	 *
	 * @param unknown $rating_form_id
	 * @return string
	 */
	public function gform_view_entries_page( $page, $rating_form_id ) {
	
		$gform_form_id = $this->get_gform_form_id( $rating_form_id );
	
		if ( $gform_form_id ) {
			return 'gf_entries&id=' . $gform_form_id;
		}
			
		return $page;
			
	}
	
	/**
	 * Gets Gravity Form based on rating form id
	 * 
	 * @param unknown $rating_form_id
	 * @return rating_form_id|NULL
	 */
	public function get_gform_form_id( $rating_form_id ) {
		
		global $wpdb;
		
		$gform_form_id = $wpdb->get_var( 'SELECT gform_form_id FROM ' . $wpdb->prefix . 'mrp_gravity_forms_form WHERE rating_form_id = ' . intval( $rating_form_id ) );
		
		if ( $gform_form_id ) {
			return $gform_form_id;
		}
		
		return null;
	}
	
	/**
	 * Gets rating form id based on a Gravity Form
	 *
	 * @param unknown $gform_form_id
	 * @return gform_form_id|NULL
	 */
	public function get_rating_form_id( $gform_form_id ) {
		
		global $wpdb;
	
		$rating_form_id = $wpdb->get_var( 'SELECT rating_form_id FROM ' . $wpdb->prefix . 'mrp_gravity_forms_form WHERE gform_form_id = ' . intval( $gform_form_id ) );
	
		if ( $rating_form_id ) {
			return $rating_form_id;
		}
	
		return null;
	}
	
	/**
	 * Gets Gravity Form entry id (also known as lead id) based on rating entry id
	 *
	 * @param unknown $rating_entry_id
	 * @return $gform_entry_id|NULL
	 */
	public function get_gform_entry_id( $rating_entry_id ) {
	
		global $wpdb;
	
		$gform_entry_id = $wpdb->get_var( 'SELECT gform_lead_id FROM ' . $wpdb->prefix . 'mrp_gravity_forms_entry WHERE rating_entry_id = ' . intval( $rating_entry_id ) );
	
		if ( $gform_entry_id ) {
			return $gform_entry_id;
		}
	
		return null;
	}
	
	/**
	 * Gets rating entry id based on Gravity Form entry id (also known as lead id)
	 *
	 * @param unknown $gform_entry_id
	 * @return $rating_entry_id|NULL
	 */
	public function get_rating_entry_id( $gform_entry_id ) {
	
		global $wpdb;
	
		$rating_entry_id = $wpdb->get_var( 'SELECT rating_entry_id FROM ' . $wpdb->prefix . 'mrp_gravity_forms_entry WHERE gform_lead_id = ' . intval( $gform_entry_id ) );
	
		if ( $rating_entry_id ) {
			return $rating_entry_id;
		}
	
		return null;
	}
	
	/**
	 * Adds Gravity Forms settings tab
	 *
	 * @param unknown $tabs
	 * @return array
	 */
	public function add_gravity_forms_settings_tab( $tabs ) {
	
		$tabs_1 = array_splice( $tabs, count( $tabs ) - 1 );
	
		return ( array) array_merge( $tabs, array( 'mrp_gravity_forms' 	=> __( 'Gravity Forms', 'mrp-gravity-forms' ) ), $tabs_1 );
	}
	
	/**
	* Adds Gravity Forms settings form
	*
	* @param unknown $current_tab
	*/
	public function add_mrp_gravity_forms_settings_form( $current_tab ) {
	
		if ( $current_tab == 'mrp_gravity_forms' ) {
			?>
			<form method="post" name="mrp_gravity_forms" action="options.php">
				<?php
				wp_nonce_field( 'update-options' );
				settings_fields( 'mrp_gravity_forms' );
				do_settings_sections( 'mrp_gravity_forms' );
				submit_button( null, 'primary', 'submit', true, null );
				?>
			</form>
			<?php
		}
	}
	
	public function has_rating_item( $form_meta ) {
	
		foreach ( $form_meta['fields'] as $field ) {
			if ( strpos( $field->type, 'rating_item_' ) !== false ) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Saves a Gravity Form
	 * 
	 * @param unknown $form_meta
	 * @param unknown $is_new
	 */
	public function after_save_form( $form_meta, $is_new ) {
		
		if ( $this->is_preview() ) {
			return;
		}
		
		if ( ! $is_new ) {
			
			global $wpdb;
			
			$gform_form_id = $form_meta['id'];
			$rating_form_id = $this->get_rating_form_id( $gform_form_id );
			
			// If a rating item has been added and the form is not yet associated with MRP, then create 
			// an associated rating form
			if ( $this->has_rating_item( $form_meta) && $rating_form_id == null ) {
				
				$results = $wpdb->insert(  $wpdb->prefix . MRP_Multi_Rating::RATING_FORM_TBL_NAME, array( 'name' => $form_meta['title'] ) );
				$rating_form_id = $wpdb->insert_id;
				
				$wpdb->insert( $wpdb->prefix . 'mrp_gravity_forms_form', array(
						'rating_form_id' => $rating_form_id,
						'gform_form_id' => $form_meta['id'],
				), array( '%d', '%d' ) );
				
			} else if ( $rating_form_id == null ) {
				return;
			}
			
			// refresh rating items in case any were deleted...
			$wpdb->delete( $wpdb->prefix . MRP_Multi_Rating::RATING_FORM_ITEM_TBL_NAME, array( 
					'rating_form_id' => $rating_form_id,
					'item_type' => 'rating-item'
			 ), array( '%d', '%s' ) );
				
			foreach ( $form_meta['fields'] as $field ) {
				
				if ( strpos( $field->type, 'rating_item_' ) !== false ) {
			 		
			 		$weight = 1; // TODO
			 		
			 		$wpdb->insert( $wpdb->prefix . MRP_Multi_Rating::RATING_FORM_ITEM_TBL_NAME,
			 				array( 
			 						'weight' => $weight, 
			 						'required' => $field->isRequired, 
			 						'rating_form_id' => $rating_form_id, 
			 						'item_id' => $field->rating_item['rating_item_id'], 
			 						'item_type' => 'rating-item' 
			 				),
			 				array( '%f', '%d', '%d', '%d', '%s' ) 
			 		);
			 		
			 	}
			 }
		}
	}

	/**
	 * Registers plugin settings
	 */
	public function register_mrp_gravity_forms_settings() {
		
		register_setting( 'mrp_gravity_forms', 'mrp_gravity_forms', array( MRP_Gravity_Forms::get_instance(), 'sanitize_mrp_gravity_forms_settings' ) );
	
		add_settings_section( 'mrp_gravity_forms_section', null, array( MRP_Gravity_Forms::get_instance(), 'section_mrp_gravity_forms_desc' ), 'mrp_gravity_forms' );
	
		add_settings_field( 'image_size', __( 'Image Display Size', 'mrp_gravity_forms' ), array( MRP_Gravity_Forms::get_instance(), 'image_size_field' ), 'mrp_gravity_forms', 'mrp_gravity_forms_section' );	
		
		$delete_associated_data = array( 
				array( 'name' => 'rating_form',		'label' => __( 'If a Gravity Form is deleted, delete it\'s associated Multi Rating Pro rating form (Note: deleting a Gravity Form will also delete all of it\'s associated entries with it).', 'mrp_gravity_forms' ) ),
				array( 'name' => 'gform_entry',		'label' => __( 'If a Gravity Form entry is deleted, delete it\'s associated Multi Rating Pro rating entry.', 'mrp_gravity_forms' ) ),
				array( 'name' => 'rating_entry', 	'label' => __( 'If a Multi Rating Pro rating entry is deleted, delete it\'s associated Gravity Form entry.', 'mrp_gravity_forms'  ) ),
		);
		
		$setting_fields = array(
				'delete_associated_data' => array(
						'title' 	=> __( 'Delete Associated Data', 'mrp_gravity_forms' ),
						'callback' 	=> 'field_checkboxes',
						'page' 		=> 'mrp_gravity_forms',
						'section' 	=> 'mrp_gravity_forms_section',
						'args' => array(
								'option_name' 	=> 'mrp_gravity_forms',
								'setting_id' 	=> 'delete_associated_data',
								'checkboxes' 	=> $delete_associated_data,
								//'description'	=> __( '', 'mrp_gravity_forms' )
						)
				)
		);
	
		
		foreach ( $setting_fields as $setting_id => $setting_data ) {
		
			// $id, $title, $callback, $page, $section, $args
			add_settings_field( $setting_id, $setting_data['title'], array( $this, $setting_data['callback'] ), $setting_data['page'], $setting_data['section'], isset( $setting_data['args'] ) ? $setting_data['args'] : array() );
		}
	}
	
	/**
	 * Defaults settings if not set
	 */
	public function load_settings() {
		
		$settings = (array) get_option( 'mrp_gravity_forms' );
		
		$settings = array_merge( array(
				'image_size' => 'thumbnail',
				'delete_associated_data' => array( 0 => 'rating_form', 1 => 'gform', 2 => 'rating_entry', 3 => 'gform_entry' )
		), $settings );
		
		update_option( 'mrp_gravity_forms', $settings );
	}

	/**
	 * Sanitize settings
	 * 
	 * @param unknown $input
	 * @return unknown
	 */
	public function sanitize_mrp_gravity_forms_settings( $input ) {
		
		if ( ! isset( $input['delete_associated_data'] ) ) {
			$input['delete_associated_data'] = array();
		}
		
		return $input;
	}
	
	/**
	 * Settings section
	 */
	public function section_mrp_gravity_forms_desc() {
	}
	
	/**
	 * Settings field
	 */
	public function image_size_field() {
		$settings = (array) get_option( 'mrp_gravity_forms' );
		$image_size = $settings['image_size'];
		?>
		<select name="mrp_gravity_forms[image_size]">
			<?php 
			$img_sizes = MRP_Utils::get_image_sizes();
			
			foreach ( $img_sizes as $img_size_name => $img_size_meta ) {
				$selected = '';
				if ( $img_size_name == $image_size ) {
					$selected = ' selected="selected"';
				}
				echo '<option value="' . $img_size_name . '" ' . $selected . '>' . $img_size_name . ' (' .  $img_size_meta['width'] . 'x' . $img_size_meta['height'] . ')</option>';
			}
			?>
		</select>
		<label><?php _e( 'Uploaded images will be automatically resized for display.', 'mrp-gravity-forms' ); ?></label>
		<?php
	}
	
	/**
	 * Checkbox setting
	 */
	function field_checkbox( $args ) {
		$settings = (array) get_option( $args['option_name' ] );
		?>
		<input type="checkbox" name="<?php echo $args['option_name']; ?>[<?php echo $args['setting_id']; ?>]" value="true" <?php checked( true, isset( $settings[$args['setting_id']] ) ? $settings[$args['setting_id']] : false , true ); ?> />
		<?php 
		if ( isset( $args['label'] ) ) { ?>
			<label><?php echo $args['label']; ?></label>
		<?php }
	}
		
	/**
	 * Checkboxes field
	 *
	 * @param unknown $args
	 */
	function field_checkboxes( $args ) {
		$settings = (array) get_option( $args['option_name' ] );
		$value = $settings[$args['setting_id']];
	
		$index = 0;
		foreach ( $args['checkboxes'] as $checkbox ) {
			
			$checked = '';
			if ( is_array( $value ) ) {
				if ( in_array( $checkbox['name'], $value ) ) {
					$checked = 'checked="checked"';
				}
			} else if ( $checkbox['name'] == $value ) {
				$checked = 'checked="checked"';
			}
			
			
			?>
			<input type="checkbox" name="<?php echo $args['option_name']; ?>[<?php echo $args['setting_id']; ?>][]" value="<?php echo $checkbox['name']; ?>" <?php echo $checked; ?> />
			<label class="checkbox-label"><?php echo $checkbox['label']; ?></label>
			<?php 
			if ( ++$index < count( $args['checkboxes'] ) ) {
				echo '<br />';
			}
		}
		
		if ( isset( $args['description'] ) ) {
			?>
			<p><?php echo $args['description']; ?></p>
			<?php
		}
	}
	
	/**
	 * Configures the settings which should be rendered on the Form Settings > Simple Add-On tab.
	 *
	 * @return array
	 */
	public function form_settings_fields( $form ) {
		
		$review_fields_map = array(
				array(
						'name'          => 'title',
						'label'         => esc_html__( 'Title', 'mrp-gravity-forms' ),
						'required'      => false,
						'field_type'    => array( 'text', 'hidden' ),
						'default_value' => $this->get_first_field_by_type( 'title', 3 ),
				),
				array(
						'name'          => 'name',
						'label'         => esc_html__( 'Name', 'mrp-gravity-forms' ),
						'required'      => false,
						'field_type'    => array( 'name', 'text', 'hidden' ),
						'default_value' => $this->get_first_field_by_type( 'name', 3 ),
				),
				array(
						'name'          => 'email',
						'label'         => esc_html__( 'Email', 'mrp-gravity-forms' ),
						'required'      => false,
						'field_type'    => array( 'email', 'hidden' ),
						'default_value' => $this->get_first_field_by_type( 'email', 3 ),
				),
				array(
						'name'          => 'comment',
						'label'         => esc_html__( 'Comment', 'mrp-gravity-forms' ),
						'required'      => false,
						'field_type'    => array( 'textarea', 'text', 'hidden' ),
						'default_value' => $this->get_first_field_by_type( 'title', 3 ),
				)	
		);
		
		return array(
				array( 
						'title'  => esc_html__( 'Multi Rating Pro Settings', 'mrp-gravity-forms' ),
						'fields' => array(
								array(
					            		'label'   => esc_html__( 'Review Fields', 'mrp-gravity-forms' ),
										'type'    => 'field_map',
										'name'    => 'mrp_review_fields_map',
										'tooltip' => esc_html__( 'You can map Multi Rating Pro review fields to Gravity Form form fields.', 'mrp-gravity-forms' ),
										'field_map' => $review_fields_map
								)
						)
				)
		);
		
	}
	
	/**
	 * Updates review field mapping
	 * 
	 * @param unknown $form_meta
	 * @param unknown $form_id
	 */
	public function update_form_settings( $form_meta, $form_id ) {
		
		$rating_form_id = $this->get_rating_form_id( $form_id );
		$form = GFAPI::get_form( $form_id );
		$review_fields_map = isset( $form['mrp-gravity-forms'] ) ? $form['mrp-gravity-forms'] : array();
		
		global $wpdb;
		
		// refresh review fields
		$wpdb->delete( $wpdb->prefix . MRP_Multi_Rating::RATING_FORM_ITEM_TBL_NAME, array(
				'rating_form_id' => $rating_form_id,
				'item_type' => 'review-field'
		), array( '%d', '%s' ) );
	
		foreach ( $review_fields_map as $field_name => $field_id ) {
			
			$field = RGFormsModel::get_field( $form, $field_id );
			
			$item_id = MRP_Multi_Rating::TITLE_REVIEW_FIELD_ID;
			if ( strpos( $field_name, 'name' ) > 0 ) {
				$item_id = MRP_Multi_Rating::NAME_REVIEW_FIELD_ID;
			} else if ( strpos( $field_name, 'email' ) > 0 ) {
				$item_id = MRP_Multi_Rating::EMAIL_REVIEW_FIELD_ID;
			} else if ( strpos( $field_name, 'comment' ) > 0 ) {
				$item_id = MRP_Multi_Rating::COMMENT_REVIEW_FIELD_ID;
			}
			
			if ( $field ) {
				$wpdb->insert( $wpdb->prefix . MRP_Multi_Rating::RATING_FORM_ITEM_TBL_NAME,
						array(
								'required' => $field->isRequired,
								'rating_form_id' => $rating_form_id,
								'item_id' => $item_id,
								'item_type' => 'review-field'
						),
						array( '%d', '%d', '%d', '%s' )
				);
				
			}
			
		}

	}
	
	/**
	 * Returns Gravity Form html
	 * 
	 * @param unknown $post_id
	 * @param unknown $rating_form_id
	 * @param unknown $rating_form_position
	 * @return html
	 */
	public function rating_form( $post_id, $rating_form_id, $rating_form_position ) {
	
		$rating_form_id = MRP_Utils::get_rating_form( $post_id );
		$gform_form_id = $this->get_gform_form_id( $rating_form_id );
	
		return gravity_form( $gform_form_id, true, true, false, '', false, 0, false );

	}
	

	/**
	 * Adds auto placement function callback for displaying Gravity Form
	 * 
	 * @param unknown $rating_form_callback
	 * @param unknown $post_id
	 * @param unknown $rating_form_id
	 * @return callback function
	 */
	public function rating_form_callback( $rating_form_callback, $post_id, $rating_form_id ) {
		
		$gform_form_id = $this->get_gform_form_id( $rating_form_id );

		if ( $gform_form_id && GFAPI::get_form( $gform_form_id ) ) {
			return array( MRP_Gravity_Forms::get_instance(), 'rating_form' );
		}
		
		return $rating_form_callback;
		
	}
	
	
	/**
	 * Displays Gravity Form fields in rating entry details
	 * 
	 * @param unknown $rating_entry
	 */
	public function gform_fields_display( $rating_entry ) {
	
		$lead_id = $this->get_gform_entry_id( $rating_entry['rating_entry_id'] );
		
		if ( ! $lead_id ) {
			return;
		}
		
		$lead = RGFormsModel::get_lead( $lead_id );
	
		$form = GFFormsModel::get_form_meta( $lead['form_id'] );
		$review_fields_map = isset( $form['mrp-gravity-forms'] ) ? $form['mrp-gravity-forms'] : array();
		
		// in case you want to change the fields order, or modift any data etc...
		$fields = (array) apply_filters( 'mrp_gform_fields_pre_display', $form['fields'], $rating_entry, $lead );
	
		foreach( $fields as $field ) {
	
			// you can remove a field from the frontend by making it admin only in the field advanced settings
			if ( $field['adminOnly'] || in_array( $field['id'], $review_fields_map ) || strpos( $field->type, 'rating_item_' ) > 0  ) {
				continue;
			}
			
			$field_html = '';
			
			switch ( $field->get_input_type() ) {
				
				case 'text':
				case 'website':
				case 'phone':
				case 'number':
				case 'date':
				case 'time':
				case 'textarea':
				case 'select':
				case 'radio':
				case 'name':
				case 'email':
					
					$field_value = RGFormsModel::get_lead_field_value( $lead, $field );
					$field_display = apply_filters( 'gform_entry_field_value', GFCommon::get_lead_field_display( $field, $field_value, $lead['currency'] ), $field, $lead, $form );
					$field_label = GFCommon::get_label( $field );
					
					$field_html = '<p class="mrp">'
							. '<label class="description">' . esc_html( $field_label ) . '</label>'
							. '<span class="value-text">' . $field_display . '</span>'
							. '</p>';
					
					break;
					
				case 'address':
					
					$field_value = RGFormsModel::get_lead_field_value( $lead, $field );
					$field_display = apply_filters( 'gform_entry_field_value', GFCommon::get_lead_field_display( $field, $field_value, $lead['currency'] ), $field, $lead, $form );
					$field_label = GFCommon::get_label( $field );
						
					$field_html = '<p class="mrp">'
							. '<label class="description">' . esc_html( $field_label ) . '</label><br />'
							. '<span class="value-text">' . $field_display . '</span>'
							. '</p>';
					
					break;
					
				case 'checkbox':
					
					$field_value = RGFormsModel::get_lead_field_value( $lead, $field );
					$field_display = apply_filters( 'gform_entry_field_value', GFCommon::get_lead_field_display( $field, $field_value, $lead['currency'] ), $field, $lead, $form );
					$field_label = GFCommon::get_label( $field );
					
					$field_html = '<p class="mrp">'
							. '<label class="description">' . esc_html( $field_label ) . '</label>'
							. '</p>';
					$field_html .= $field_display;
					
					break;
					
				case 'fileupload':
					
					$image_file_mime_types = array(
							'image/jpeg',
							'image/gif',
							'image/png',
							'image/bmp',
							'image/tiff',
							'image/x-icon'
					);
					
					$field_value = RGFormsModel::get_lead_field_value( $lead, $field );
					$file_mime_type = wp_check_filetype( $field_value );
					
					if ( in_array( $file_mime_type['type'], $image_file_mime_types) ) {
						
						$image_size_meta = $this->get_image_size_meta();
						
						$field_label = GFCommon::get_label( $field );
						
						// TODO move inline style to a frontend.css file
						$field_html = '<p class="mrp">'
								. '<label class="description">' . esc_html( $field_label ) . '</label>'
								. '<img style="display: block;" class="fileupload" width="' . $image_size_meta['width'] . '" height="' . $image_size_meta['height'] . '" href=\"' . $field_value . ' src="' . $field_value . '"></a>'
								. '</p>';
						
					}
					break;
			
				default :
					// do nothing
					break;
					
			}
			
			echo apply_filters( 'mrp_gform_field_html', $field_html, $form, $field, $lead );
		}
	}
	
	/**
	 * Checks for plugin updates in the WordPress admin
	 */
	function plugin_updater() {
	
		if ( class_exists( 'MRP_Plugin_Updater' ) ) {
	
			// retrieve the license from the database
			$license_settings = (array) get_option( MRP_Multi_Rating::LICENSE_SETTINGS );
			$license_key = isset( $license_settings['mrp_gform_license'] ) ? trim( $license_settings['mrp_gform_license'] ) : '';
	
			// setup the updater
			new MRP_Plugin_Updater( 'http://multiratingpro.com', __FILE__, array(
					'version' 	=> '1.0.1', 						// current version number
					'license' 	=> $license_key, 					// license key (used get_option above to retrieve from DB)
					'item_name' => urldecode( 'Gravity Forms Add-On' ),	// name of this plugin
					'author' 	=> 'Daniel Powney'  				// author of this plugin
			) );
				
		}
	}
	
	/**
	 *
	 * @param unknown $license_items
	 * @return unknown
	*/
	public function license_items( $license_items ) {
		$license_items['mrp_gform_license'] = urlencode( 'Gravity Forms Add-On' );
		return $license_items;
	}
	
	/**
	 *
	 * @param unknown $setting_sections
	 * @return unknown
	*/
	public function license_settings_sections( $setting_sections ) {
	
		array_push( $setting_sections, array(
				'id' => 'section_mrp_gform_license',
				'title' => __( 'Gravity Forms Add-On', 'mrp-gravity-forms' ),
				'callback' => 'section_license_desc',
				'page' => MRP_Multi_Rating::SETTINGS_PAGE_SLUG . '&tab=' . MRP_Multi_Rating::LICENSES_SETTINGS_TAB
		) );
	
		return $setting_sections;
	
	}
	
	/**
	 *
	 * @param unknown $setting_fields
	 * @return unknown
	*/
	public function license_setting_fields( $setting_fields ) {
	
		$setting_fields['mrp_gform_license'] = array(
				'title' 	=> __( 'License Key', 'mrp-gravity-forms' ),
				'callback' 	=> 'field_license_key',
				'page' 		=> MRP_Multi_Rating::SETTINGS_PAGE_SLUG . '&tab=' . MRP_Multi_Rating::LICENSES_SETTINGS_TAB,
				'section' 	=> 'section_mrp_gform_license',
				'args' => array(
						'option_name' 	=> MRP_Multi_Rating::LICENSE_SETTINGS,
						'setting_id' 	=> 'mrp_gform_license',
						'label' 		=> __( 'Enter your license key.', 'mrp-gravity-forms' )
				),
				'item_name' => urlencode( 'Gravity Forms Add-On' ), // the name of our product in EDD
		);
	
		return $setting_fields;
	
	}
	
}

/**
 * Fires after gform is loaded
 */
function mrp_gform_loaded() {
	if ( ! method_exists( 'GFForms', 'include_addon_framework' ) ) {
		return;
	}

	GFForms::include_addon_framework();

	GFAddOn::register( 'MRP_Gravity_Forms' );
}
// init plugin only if MRP is enabled
add_action( 'gform_loaded', 'mrp_gform_loaded', 5 );


/**
 * Activate plugin
 */
function mrp_gravity_forms_activate_plugin() {

	if ( is_admin() && ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX ) ) {
		MRP_Gravity_Forms::activate_plugin();
	}
}
register_activation_hook( __FILE__, 'mrp_gravity_forms_activate_plugin');
